#!/usr/bin/perl -w

# VersaCount
# Copyright Charlie Kim 2007-2010

# This program is free software: you can redistribute it and/or 
# modify it under the terms of the GNU General Public License as 
# publisehd by the Free Software Foundation, either version 3 of 
# the License, or (at your option) any later version.

# This program is distributed in the hope that it will be useful, 
# but WITHOUT ANY WARRANTY; without even the implied warranty of 
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the 
# GNU General Public License at http://www.gnu.org/licenses/ for 
# more details.

# v0.1 - modified parasite counter v0.62 to make it more universal 
#        and customizable for all cell counting
# v0.2 - fixed updating of run tab to reflect options tab, added 
#        sounds for cell6 and cell7
# v0.3 - added 2nd alert, alert configurability
# v0.4 - equation editability
# v0.5 - help tab added
# v0.6 - updates to interface - changes in button sizes
# v1.0 - updated name to VersaCount for first public release

use strict;

use Tk;
use Tk::NumEntryPlain;
use Tk::LabFrame;
use Tk::ROText;
use Tk::NoteBook;
use Win32::Sound;
use POSIX;

#print "OS: $^O\n";

#if ($^O =~ /MSWin32/i) {
#    Module::Load::Conditional::can_load(modules => { 'Win32::Sound' => undef } );
#}
#else {
#    Module::Load::Conditional::can_load(modules => { 'Audio::Data' => undef } );
#    Module::Load::Conditional::can_load(modules => { 'Audio::Play' => undef } );
#}

&main;

sub main {
    my $mw = MainWindow->new(-title => 'VersaCount 1.0');

    my $iconData = <<EOF;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    my $image = $mw->Photo(-data => $iconData);
    $mw->Icon( -image => $image );

# settings
    my $addCell2 = 'Insert';
    my $addCell3 = 'comma';
    my $addCell4 = 'Delete';
    my $addCell5 = 'Return';
    my $add1cell1 = 'End';
    my $add2cell1 = 'Down';
    my $add3cell1 = 'Next';
    my $add4cell1 = 'Left';
    my $add5cell1 = 'Win_L';
    my $add6cell1 = 'Right';
    my $add7cell1 = 'Home';
    my $add8cell1 = 'Up';
    my $add9cell1 = 'Prior';
    my $addCell6 = 'minus';
    my $addCell7 = 'plus';

    my $subCell1 = '1';
    my $subCell2 = '2';
    my $subCell3 = '3';
    my $subCell4 = '4';
    my $subCell5 = '5';
    my $subCell6 = '6';
    my $subCell7 = '7';

    my $soundToggle = 1;
    my $soundCell1 = 'VersaCountSettings/beep1.wav';
    my $soundCell2 = 'VersaCountSettings/beep2.wav';
    my $soundCell3 = 'VersaCountSettings/beep3.wav';
    my $soundCell4 = 'VersaCountSettings/beep4.wav';
    my $soundCell5 = 'VersaCountSettings/beep5.wav';
    my $soundCell6 = 'VersaCountSettings/beep6.wav';
    my $soundCell7 = 'VersaCountSettings/beep7.wav';
    my $soundAlert1 = 'VersaCountSettings/alert1.wav';
    my $soundAlert2 = 'VersaCountSettings/alert2.wav';

    my $cell1 = 0;
    my $cell2 = 0;
    my $cell3 = 0;
    my $cell4 = 0;
    my $cell5 = 0;
    my $cell6 = 0;
    my $cell7 = 0;
    my $totalCellCount = 0;

    my $cell1label = 'Cell1';
    my $cell2label = 'Cell2';
    my $cell3label = 'Cell3';
    my $cell4label = 'Cell4';
    my $cell5label = 'Cell5';
    my $cell6label = 'Cell6';
    my $cell7label = 'Cell7';

    my $soundAlert1val = 100;
    my $soundAlert2val = 500;

# define equations
    my $equation1label = 'Equation1';
    my $equation2label = 'Equation2';
    my $equation1 = '$cell1';
    my $equation2 = '$cell1';

    my ($cell1count,
	$cell2count,
	$cell3count,
	$cell4count,
	$cell5count,
	$cell6count,
	$cell7count,
	$cell1label_b,
	$cell2label_b,
	$cell3label_b,
	$cell4label_b,
	$cell5label_b,
	$cell6label_b,
	$cell7label_b,
	$addcell1label,
	$addcell2label,
	$addcell3label,
	$addcell4label,
	$addcell5label,
	$addcell6label,
	$addcell7label,
	$subcell1label,
	$subcell2label,
	$subcell3label,
	$subcell4label,
	$subcell5label,
	$subcell6label,
	$subcell7label,
	$addcell2keylabel,
	$addcell3keylabel,
	$addcell4keylabel,
	$addcell5keylabel,
	$addcell6keylabel,
	$addcell7keylabel,
	$subcell1keylabel,
	$subcell2keylabel,
	$subcell3keylabel,
	$subcell4keylabel,
	$subcell5keylabel,
	$subcell6keylabel,
	$subcell7keylabel,
	$totalCellCount_e,
	$equation1label_e,
	$equation2label_e,
	$equation1val_e,
	$equation2val_e,
	$logwin,
	$book,
	$keyTestWin,
	);

    # labels
    my $keybind_cell2label;

# file handles for linux sound files
    my ($audioDataCell1,
	$audioDataCell2,
	$audioDataCell3,
	$audioDataCell4,
	$audioDataCell5,
	$audioDataCell6,
	$audioDataCell7,
	$audioDataAlert1,
	$audioDataAlert2);
    my $svr; # for Audio::Play;
    if ($^O !~ /MSWin32/i) {
	$svr = Audio::Play->new;
    }

# Summarize parameters
    my @parameters = (\$mw,
		      \$cell1count,
		      \$cell2count,
		      \$cell3count,
		      \$cell4count,
		      \$cell5count,
		      \$cell6count,
		      \$cell7count,
		      \$totalCellCount,
		      \$totalCellCount_e,
		      \$equation1,
		      \$equation2,
		      \$equation1label,
		      \$equation2label,
		      \$equation1val_e,
		      \$equation2val_e,
		      \$cell1,
		      \$cell2,
		      \$cell3,
		      \$cell4,
		      \$cell5,
		      \$cell6,
		      \$cell7,
		      \$cell1label,
		      \$cell2label,
		      \$cell3label,
		      \$cell4label,
		      \$cell5label,
		      \$cell6label,
		      \$cell7label,
		      \$cell1label_b,
		      \$cell2label_b,
		      \$cell3label_b,
		      \$cell4label_b,
		      \$cell5label_b,
		      \$cell6label_b,
		      \$cell7label_b,
		      \$logwin,
		      \$book,
		      \$keyTestWin,
		      \$addCell2,
		      \$addCell3,
		      \$addCell4,
		      \$addCell5,
		      \$add1cell1,
		      \$add2cell1,
		      \$add3cell1,
		      \$add4cell1,
		      \$add5cell1,
		      \$add6cell1,
		      \$add7cell1,
		      \$add8cell1,
		      \$add9cell1,
		      \$addCell6,
		      \$addCell7,
		      \$subCell1,
		      \$subCell2,
		      \$subCell3,
		      \$subCell4,
		      \$subCell5,
		      \$subCell6,
		      \$subCell7,
		      \$soundToggle,
		      \$soundCell1,
		      \$soundCell2,
		      \$soundCell3,
		      \$soundCell4,
		      \$soundCell5,
		      \$soundCell6,
		      \$soundCell7,
		      \$soundAlert1,
		      \$soundAlert2,
		      \$soundAlert1val,
		      \$soundAlert2val,
		      \$audioDataCell1,
		      \$audioDataCell2,
		      \$audioDataCell3,
		      \$audioDataCell4,
		      \$audioDataCell5,
		      \$audioDataCell6,
		      \$audioDataCell7,
		      \$audioDataAlert1,
		      \$audioDataAlert2,
		      \$svr,
		      );

    my @cellLabels = (
		      \$cell1label,
		      \$cell2label,
		      \$cell3label,
		      \$cell4label,
		      \$cell5label,
		      \$cell6label,
		      \$cell7label,
		      \$cell1label_b,
		      \$cell2label_b,
		      \$cell3label_b,
		      \$cell4label_b,
		      \$cell5label_b,
		      \$cell6label_b,
		      \$cell7label_b,
		      \$addcell1label,
		      \$addcell2label,
		      \$addcell3label,
		      \$addcell4label,
		      \$addcell5label,
		      \$addcell6label,
		      \$addcell7label,
		      \$subcell1label,
		      \$subcell2label,
		      \$subcell3label,
		      \$subcell4label,
		      \$subcell5label,
		      \$subcell6label,
		      \$subcell7label,
		      \$addCell2,
		      \$addCell3,
		      \$addCell4,
		      \$addCell5,
		      \$addCell6,
		      \$addCell7,
		      \$subCell1,
		      \$subCell2,
		      \$subCell3,
		      \$subCell4,
		      \$subCell5,
		      \$subCell6,
		      \$subCell7,
		      \$addcell2keylabel,
		      \$addcell3keylabel,
		      \$addcell4keylabel,
		      \$addcell5keylabel,
		      \$addcell6keylabel,
		      \$addcell7keylabel,
		      \$subcell1keylabel,
		      \$subcell2keylabel,
		      \$subcell3keylabel,
		      \$subcell4keylabel,
		      \$subcell5keylabel,
		      \$subcell6keylabel,
		      \$subcell7keylabel,
		      \$equation1label_e,
		      \$equation2label_e,
		      \$equation1label,
		      \$equation2label
		      );
    
# automatically load previous settings

    &autoLoadSettings(@parameters);

# font options
#    my $bigFont = 'systemfixed 20';
#    my $bigFont = 'system 20'; # looks bad on win32
#    my $bigFont = 'ansifixed 20'; # looks like systemfixed on win32
#    my $bigFont = 'ansi 20'; # looks like systemfixed on win32
#    my $bigFont = 'device 20'; # looks like systemfixed on win32
#    my $bigFont = 'oemfixed 20'; # looks like systemfixed on win32
    my $bigFont = 'fixed 20'; # looks the same as systemfixed on win32

# set up notebook
    
    $book = $mw->NoteBook()->pack(-fill => 'both',
                                  -expand => 1,
                                  );
    
    my $runTab = $book->add( "Counter",
                             -label => "Counter",
                             -raisecmd => sub { $mw->focus;
                                                &checkSettings(@parameters);
                                                &updateButtons(@cellLabels);
                                            },
                             );
    my $optionTab = $book->add( "Options",
                                -label => "Options",
                                -raisecmd => sub { $mw->focus; },
                                );
    my $keyTab = $book->add( "Keystroke Test",
                             -label => "Keystroke Test",
                             -raisecmd => sub { $mw->focus; },
                             );
    
    my $helpTab = $book->add( "Help",
                              -label => "Help",
                              -raisecmd => sub { $mw->focus; },
                              );
    

# run tab frames

    my $runTabLeft = $runTab->Frame->grid(-row => 0,
					  -column => 0,
					  );
    my $runTabRight = $runTab->Frame->grid(-row => 0,
					   -column => 1,
					   );

    my $instructionFr = $runTabLeft->LabFrame(-label => 'Instructions',
					      -labelside => 'acrosstop',
					      )->pack(-fill => 'x');
    my $countFr = $runTabLeft->LabFrame(-label => 'Counts',
					-labelside => 'acrosstop',
					)->pack(-fill => 'x');

    my $commandFrTop = $runTabRight->Frame()->pack(-fill => 'x');
    my $logFr = $runTabRight->LabFrame(-label => 'Keystroke Log',
				       -labelside => 'acrosstop',
				       )->pack(-fill => 'both',
					  );
    my $commandFrBot = $runTabRight->Frame()->pack(-fill => 'x');


# options tab frames

    my $bindingFr = $optionTab->LabFrame(-label => 'Labels and Key Bindings',
					 -labelside => 'acrosstop',
					 )->pack(-fill => 'x');
    my $soundFr = $optionTab->LabFrame(-label => 'Audio Feedback',
				       -labelside => 'acrosstop',
				       )->pack(-fill => 'x');
    my $optionFileFr = $optionTab->Frame->pack(-fill => 'x');


# cell labels

    $bindingFr->Label(-text => "Cell 1 Name")->grid(-row => 0,
						    -column => 0,
						    -sticky => 'e',
						    );
    $bindingFr->Entry(-textvariable => \$cell1label,
		      -width => 10,
		      )->grid(-row => 0,
			      -column => 1,
			      -sticky => 'w',
			      );

    $bindingFr->Label(-text => "Cell 2 Name")->grid(-row => 1,
						    -column => 0,
						    -sticky => 'e',
						    );
    $bindingFr->Entry(-textvariable => \$cell2label,
		      -width => 10,
		      )->grid(-row => 1,
			      -column => 1,
			      -sticky => 'w',
			      );

    $bindingFr->Label(-text => "Cell 3 Name")->grid(-row => 2,
						    -column => 0,
						    -sticky => 'e',
						    );
    $bindingFr->Entry(-textvariable => \$cell3label,
		      -width => 10,
		      )->grid(-row => 2,
			      -column => 1,
			      -sticky => 'w',
			      );

    $bindingFr->Label(-text => "Cell 4 Name")->grid(-row => 3,
						    -column => 0,
						    -sticky => 'e',
						    );
    $bindingFr->Entry(-textvariable => \$cell4label,
		      -width => 10,
		      )->grid(-row => 3,
			      -column => 1,
			      -sticky => 'w',
			      );

    $bindingFr->Label(-text => "Cell 5 Name")->grid(-row => 4,
						    -column => 0,
						    -sticky => 'e',
						    );
    $bindingFr->Entry(-textvariable => \$cell5label,
		      -width => 10,
		      )->grid(-row => 4,
			      -column => 1,
			      -sticky => 'w',
			      );

    $bindingFr->Label(-text => "Cell 6 Name")->grid(-row => 5,
						    -column => 0,
						    -sticky => 'e',
						    );
    $bindingFr->Entry(-textvariable => \$cell6label,
		      -width => 10,
		      )->grid(-row => 5,
			      -column => 1,
			      -sticky => 'w',
			      );

    $bindingFr->Label(-text => "Cell 7 Name")->grid(-row => 6,
						    -column => 0,
						    -sticky => 'e',
						    );
    $bindingFr->Entry(-textvariable => \$cell7label,
		      -width => 10,
		      )->grid(-row => 6,
			      -column => 1,
			      -sticky => 'w',
			      );

# alert values

    $bindingFr->Label(-text => "Alert 1 #")->grid(-row => 7,
					       -column => 0,
					       -sticky => 'e',
					       );
    $bindingFr->Entry(-textvariable => \$soundAlert1val,
		      -width => 10,
		      )->grid(-row => 7,
			      -column => 1,
			      -sticky => 'w',
			      );
    
    $bindingFr->Label(-text => "Alert 2 #")->grid(-row => 8,
					       -column => 0,
					       -sticky => 'e',
					       );
    $bindingFr->Entry(-textvariable => \$soundAlert2val,
		      -width => 10,
		      )->grid(-row => 8,
			      -column => 1,
			      -sticky => 'w',
			      );

# equation labels

    $bindingFr->Label(-text => "Equation 1 Name")->grid(-row => 9,
							-column => 0,
							-sticky => 'e',
							);
    $bindingFr->Entry(-textvariable => \$equation1label,
		      -width => 10,
		      )->grid(-row => 9,
			      -column => 1,
			      -sticky => 'w',
			      );
    
    $bindingFr->Label(-text => "Equation 2 Name")->grid(-row => 10,
							-column => 0,
							-sticky => 'e',
							);
    $bindingFr->Entry(-textvariable => \$equation2label,
		      -width => 10,
		      )->grid(-row => 10,
			      -column => 1,
			      -sticky => 'w',
			      );
    
# equations

    $bindingFr->Label(-text => "Equation 1")->grid(-row => 11,
						   -column => 0,
						   -sticky => 'e',
						   );
    $bindingFr->Entry(-textvariable => \$equation1,
		      -width => 64,
		      )->grid(-row => 11,
			      -column => 1,
			      -columnspan => 7,
			      -sticky => 'w',
			      );

    $bindingFr->Label(-text => "Equation 2")->grid(-row => 12,
						   -column => 0,
						   -sticky => 'e',
						   );
    $bindingFr->Entry(-textvariable => \$equation2,
		      -width => 64,
		      )->grid(-row => 12,
			      -column => 1,
			      -columnspan => 7,
			      -sticky => 'w',
			      );


		      
# key bindings

    $bindingFr->Label(-text => "Add Cell 2")->grid(-row => 0,
						   -column => 3,
						   -sticky => 'e',
						   );
    $bindingFr->Entry(-textvariable => \$addCell2,
		      -width => 10,
		      )->grid(-row => 0,
			      -column => 4,
			      -sticky => 'w',
			      );
    $bindingFr->Label(-text => "Add Cell 3")->grid(-row => 1,
						   -column => 3,
						   -sticky => 'e',
						   );
    $bindingFr->Entry(-textvariable => \$addCell3,
		      -width => 10,
		      )->grid(-row => 1,
			      -column => 4,
			      -sticky => 'w',
			      );
    $bindingFr->Label(-text => "Add Cell 4")->grid(-row => 2,
						   -column => 3,
						   -sticky => 'e',
						   );
    $bindingFr->Entry(-textvariable => \$addCell4,
		      -width => 10,
		      )->grid(-row => 2,
			      -column => 4,
			      -sticky => 'w',
			      );
    $bindingFr->Label(-text => "Add Cell 5")->grid(-row => 3,
						   -column => 3,
						   -sticky => 'e',
						   );
    $bindingFr->Entry(-textvariable => \$addCell5,
		      -width => 10,
		      )->grid(-row => 3,
			      -column => 4,
			      -sticky => 'w',
			      );
    $bindingFr->Label(-text => "Add Cell 6")->grid(-row => 4,
						   -column => 3,
						   -sticky => 'e',
						   );
    $bindingFr->Entry(-textvariable => \$addCell6,
		      -width => 10,
		      )->grid(-row => 4,
			      -column => 4,
			      -sticky => 'w',
			      );
    $bindingFr->Label(-text => "Add Cell 7")->grid(-row => 5,
						   -column => 3,
						   -sticky => 'e',
						   );
    $bindingFr->Entry(-textvariable => \$addCell7,
		      -width => 10,
		      )->grid(-row => 5,
			      -column => 4,
			      -sticky => 'w',
			      );

    $bindingFr->Label(-text => "Subtract Cell 1")->grid(-row => 6,
							-column => 3,
							-sticky => 'e',
							);
    $bindingFr->Entry(-textvariable => \$subCell1,
		      -width => 10,
		      )->grid(-row => 6,
			      -column => 4,
			      -sticky => 'w',
			      );
    
    $bindingFr->Label(-text => "Subtract Cell 2")->grid(-row => 7,
							-column => 3,
							-sticky => 'e',
							);
    $bindingFr->Entry(-textvariable => \$subCell2,
		      -width => 10,
		      )->grid(-row => 7,
			      -column => 4,
			      -sticky => 'w',
			      );

    $bindingFr->Label(-text => "Subtract Cell 3")->grid(-row => 8,
							-column => 3,
							-sticky => 'e',
							);
    $bindingFr->Entry(-textvariable => \$subCell3,
		      -width => 10,
		      )->grid(-row => 8,
			      -column => 4,
			      -sticky => 'w',
			      );

    $bindingFr->Label(-text => "Subtract Cell 4")->grid(-row => 9,
							-column => 3,
							-sticky => 'e',
							);
    $bindingFr->Entry(-textvariable => \$subCell4,
		      -width => 10,
		      )->grid(-row => 9,
			      -column => 4,
			      -sticky => 'w',
			      );

    $bindingFr->Label(-text => "Subtract Cell 5")->grid(-row => 10,
							-column => 3,
							-sticky => 'e',
							);
    $bindingFr->Entry(-textvariable => \$subCell5,
		      -width => 10,
		      )->grid(-row => 10,
			      -column => 4,
			      -sticky => 'w',
			      );

    $bindingFr->Label(-text => "Subtract Cell 6")->grid(-row => 9,
							-column => 6,
							-sticky => 'e',
							);
    $bindingFr->Entry(-textvariable => \$subCell6,
		      -width => 10,
		      )->grid(-row => 9,
			      -column => 7,
			      -sticky => 'w',
			      );

    $bindingFr->Label(-text => "Subtract Cell 7")->grid(-row => 10,
							-column => 6,
							-sticky => 'e',
							);
    $bindingFr->Entry(-textvariable => \$subCell7,
		      -width => 10,
		      )->grid(-row => 10,
			      -column => 7,
			      -sticky => 'w',
			      );



    # 2nd column, add uninfecteds
    $bindingFr->Label(-text => "Add 1 Cell 1")->grid(-row => 0,
						     -column => 6,
						     -sticky => 'e',
						     );
    $bindingFr->Entry(-textvariable => \$add1cell1,
		      -width => 10,
		      )->grid(-row => 0,
			      -column => 7,
			      -sticky => 'w',
			      );
    $bindingFr->Label(-text => "Add 2 Cell 1")->grid(-row => 1,
						     -column => 6,
						     -sticky => 'e',
						     );
    $bindingFr->Entry(-textvariable => \$add2cell1,
		      -width => 10,
		      )->grid(-row => 1,
			      -column => 7,
			      -sticky => 'w',
			      );
    $bindingFr->Label(-text => "Add 3 Cell 1")->grid(-row => 2,
						     -column => 6,
						     -sticky => 'e',
						     );
    $bindingFr->Entry(-textvariable => \$add3cell1,
		      -width => 10,
		      )->grid(-row => 2, 
			      -column => 7,
			      -sticky => 'w',
			      );
    $bindingFr->Label(-text => "Add 4 Cell 1")->grid(-row => 3,
						     -column => 6,
						     -sticky => 'e',
						     );
    $bindingFr->Entry(-textvariable => \$add4cell1,
		      -width => 10,
		      )->grid(-row => 3,
			      -column => 7,
			      -sticky => 'w',
			      );
    $bindingFr->Label(-text => "Add 5 Cell 1")->grid(-row => 4,
						     -column => 6,
						     -sticky => 'e',
						     );
    $bindingFr->Entry(-textvariable => \$add5cell1,
		      -width => 10,
		      )->grid(-row => 4,
			      -column => 7,
			      -sticky => 'w',
			      );
    $bindingFr->Label(-text => "Add 6 Cell 1")->grid(-row => 5,
						     -column => 6,
						     -sticky => 'e',
						     );
    $bindingFr->Entry(-textvariable => \$add6cell1,
		      -width => 10,
		      )->grid(-row => 5,
			      -column => 7,
			      -sticky => 'w',
			      );
    $bindingFr->Label(-text => "Add 7 Cell 1")->grid(-row => 6,
						     -column => 6,
						     -sticky => 'e',
						     );
    $bindingFr->Entry(-textvariable => \$add7cell1,
		      -width => 10,
		      )->grid(-row => 6,
			      -column => 7,
			      -sticky => 'w',
			      );
    $bindingFr->Label(-text => "Add 8 Cell 1")->grid(-row => 7,
						     -column => 6,
						     -sticky => 'e',
						     );
    $bindingFr->Entry(-textvariable => \$add8cell1,
		      -width => 10,
		      )->grid(-row => 7,
			      -column => 7,
			      -sticky => 'w',
			      );
    $bindingFr->Label(-text => "Add 9 Cell 1")->grid(-row => 8,
						     -column => 6,
						     -sticky => 'e',
						     );
    $bindingFr->Entry(-textvariable => \$add9cell1,
		      -width => 10,
		      )->grid(-row => 8,
			      -column => 7,
			      -sticky => 'w',
			      );

    $bindingFr->Label()->grid(-row => 0,
			      -column => 2,
			      -padx => 5,
			      );

    $bindingFr->Label()->grid(-row => 0,
			      -column => 5,
			      -padx => 5,
			      );

# sounds

    $soundFr->Label(-text => "Cell 1 Sound File")->grid(-row => 0,
							-column => 0,
							-sticky => 'e',
							);
    my $soundCell1_e = $soundFr->Entry(-textvariable => \$soundCell1,
				    -width => 20,
				    )->grid(-row => 0,
					    -column => 1,
					    -sticky => 'e',
					    );
    $soundFr->Button(-text => 'Browse',
		     -command => [ \&selectsinglefile, \$soundCell1_e, \$mw ],
		     )->grid(-row => 0,
			     -column => 2,
			     );

    $soundFr->Button(-text => 'Test',
		     -command => sub { Win32::Sound::Play($soundCell1) },	     
		     )->grid(-row => 0,
			     -column => 3,
			     );

    $soundFr->Label(-text => "Cell 2 Sound File")->grid(-row => 1,
							-column => 0,
							-sticky => 'e',
							);
    my $soundCell2_e = $soundFr->Entry(-textvariable => \$soundCell2,
				       -width => 20,
				       )->grid(-row => 1,
					       -column => 1,
					       -sticky => 'e',
					       );
    $soundFr->Button(-text => 'Browse',
		     -command => [ \&selectsinglefile, \$soundCell2_e, \$mw ],
		     )->grid(-row => 1,
			     -column => 2,
			     );
    $soundFr->Button(-text => 'Test',
		     -command => sub { Win32::Sound::Play($soundCell2) },
		     )->grid(-row => 1,
			     -column => 3,
			     );
    
    $soundFr->Label(-text => "Cell 3 Sound File")->grid(-row => 2,
							-column => 0,
							-sticky => 'e',
							);
    my $soundCell3_e = $soundFr->Entry(-textvariable => \$soundCell3,
				       -width => 20,
				       )->grid(-row => 2,
					       -column => 1,
					       -sticky => 'e',
					       );
    $soundFr->Button(-text => 'Browse',
		     -command => [ \&selectsinglefile, \$soundCell3_e, \$mw ],
		     )->grid(-row => 2,
			     -column => 2,
			     );
    $soundFr->Button(-text => 'Test',
		     -command => sub { Win32::Sound::Play($soundCell3) },
		     )->grid(-row => 2,
			     -column => 3,
			     );
    
    $soundFr->Label(-text => "Cell 4 Sound File")->grid(-row => 3,
							-column => 0,
							-sticky => 'e',
							);
    my $soundCell4_e = $soundFr->Entry(-textvariable => \$soundCell4,
				       -width => 20,
				       )->grid(-row => 3,
					       -column => 1,
					       -sticky => 'e',
					       );
    $soundFr->Button(-text => 'Browse',
		     -command => [ \&selectsinglefile, \$soundCell4_e, \$mw ],
		     )->grid(-row => 3,
			     -column => 2,
			     );
    $soundFr->Button(-text => 'Test',
		     -command => sub { Win32::Sound::Play($soundCell4) },
		     )->grid(-row => 3,
			     -column => 3,
			     );
    
    $soundFr->Label(-text => "Cell 5 Sound File")->grid(-row => 0,
							 -column => 5,
							 -sticky => 'e',
							 );
    
    my $soundCell5_e = $soundFr->Entry(-textvariable => \$soundCell5,
				       -width => 20,
				       )->grid(-row => 0,
					       -column => 6,
					       -sticky => 'e',
					       );
    
    $soundFr->Button(-text => 'Browse',
		     -command => [ \&selectsinglefile, \$soundCell5_e, \$mw ],
		     )->grid(-row => 0,
			     -column => 7,
			     );
    
    $soundFr->Button(-text => 'Test',
		     -command => sub { Win32::Sound::Play($soundCell5) },
		     )->grid(-row => 0,
			     -column => 8,
			     );

    $soundFr->Label(-text => "Cell 6 Sound File")->grid(-row => 1,
							-column => 5,
							-sticky => 'e',
							);
    
    my $soundCell6_e = $soundFr->Entry(-textvariable => \$soundCell6,
				       -width => 20,
				       )->grid(-row => 1,
					       -column => 6,
					       -sticky => 'e',
					       );
    
    $soundFr->Button(-text => 'Browse',
		     -command => [ \&selectsinglefile, \$soundCell6_e, \$mw ],
		     )->grid(-row => 1,
			     -column => 7,
			     );
    
    $soundFr->Button(-text => 'Test',
		     -command => sub { Win32::Sound::Play($soundCell6) },
		     )->grid(-row => 1,
			     -column => 8,
			     );
    
    $soundFr->Label(-text => "Cell 7 Sound File")->grid(-row => 2,
							-column => 5,
							-sticky => 'e',
							);
    
    my $soundCell7_e = $soundFr->Entry(-textvariable => \$soundCell7,
				       -width => 20,
				       )->grid(-row => 2,
					       -column => 6,
					       -sticky => 'e',
					       );
    
    $soundFr->Button(-text => 'Browse',
		     -command => [ \&selectsinglefile, \$soundCell7_e, \$mw ],
		     )->grid(-row => 2,
			     -column => 7,
			     );
    
    $soundFr->Button(-text => 'Test',
		     -command => sub { Win32::Sound::Play($soundCell7) },
		     )->grid(-row => 2,
			     -column => 8,
			     );
    
    # sound alert 1
    $soundFr->Label(-text => 'Alert 1 Sound File')->grid(-row => 3,
							 -column => 5,
							 -sticky => 'e',
							 );
    
    
    my $soundAlert1_e = $soundFr->Entry(-textvariable => \$soundAlert1,
					-width => 20,
					)->grid(-row => 3,
						-column => 6,
						-sticky => 'e',
						);
    
    $soundFr->Button(-text => 'Browse',
		     -command => [ \&selectsinglefile, \$soundAlert1_e, \$mw ],
		     )->grid(-row => 3,
			     -column => 7,
			     );
    
    $soundFr->Button(-text => 'Test',
		     -command => sub { Win32::Sound::Play($soundAlert1) if ($^O =~ /MSWin32/i);
		       $svr->play($audioDataAlert1) if ($^O !~ /MSWin32/);
		   },
		     )->grid(-row => 3,
			     -column => 8,
			     );
    # sound alert 2
    $soundFr->Label(-text => 'Alert 2 Sound File')->grid(-row => 4,
							 -column => 5,
							 -sticky => 'e',
							 );
    
    
    my $soundAlert2_e = $soundFr->Entry(-textvariable => \$soundAlert2,
					-width => 20,
					)->grid(-row => 4,
						-column => 6,
						-sticky => 'e',
						);
    
    $soundFr->Button(-text => 'Browse',
		     -command => [ \&selectsinglefile, \$soundAlert2_e, \$mw ],
		     )->grid(-row => 4,
			     -column => 7,
			     );
    
    $soundFr->Button(-text => 'Test',
		     -command => sub { Win32::Sound::Play($soundAlert2) if ($^O =~ /MSWin32/i);
		       $svr->play($audioDataAlert2) if ($^O !~ /MSWin32/);
		   },
		     )->grid(-row => 4,
			     -column => 8,
			       );
    
    # spacer
    $soundFr->Label()->grid(-row => 0,
			    -column => 4,
			    -padx => 5,
			    );
    
    
    
    $soundFr->Label(-text => 'Use Sound')->grid(-row => 4,
						-column => 0,
						);
    $soundFr->Checkbutton(-variable => \$soundToggle)->grid(-row => 4,
							    -column => 1,
							    -sticky => 'w',
							    );
    
# keystroke test frames

    my $keytestFr = $keyTab->LabFrame(-label => 'Keystroke Test',
				      -labelside => 'acrosstop',
				      )->pack(-fill => 'x');



# Instructions Frame
    $instructionFr->Label(-text => "Press the label buttons in order to subtract counts.  Additional hot keys:")->pack;

    my $insSubFr = $instructionFr->Frame()->pack;

    $insSubFr->Label()->grid(-row => 0,
			    -column => 2,
			    -padx => 20,
			    );

    $insSubFr->Label(-text => 'See Options')->grid(-row => 0, -column => 0);
    $addcell1label = $insSubFr->Label(-text => "Add 1-9 $cell1label")->grid(-row => 0,
									    -column => 1,
									    -sticky => 'w');
    $insSubFr->Label()->grid(-row => 0, -column => 2, -padx => 5);
    $subcell1keylabel = $insSubFr->Label(-text => $subCell1)->grid(-row => 0, -column => 3);
    $subcell1label = $insSubFr->Label(-text => "Subtract 1 $cell1label")->grid(-row => 0,
									       -column => 4,
									       -sticky => 'w');
    
    $addcell2keylabel = $insSubFr->Label(-text => $addCell2)->grid(-row => 1, -column => 0);
    $addcell2label = $insSubFr->Label(-text => "Add 1 $cell2label")->grid(-row => 1,
									  -column => 1,
									  -sticky => 'w');
    $subcell2keylabel = $insSubFr->Label(-text => $subCell2)->grid(-row => 1, -column => 3);
    $subcell2label = $insSubFr->Label(-text => "Subtract 1 $cell2label")->grid(-row => 1,
									       -column => 4,
									       -sticky => 'w');
    
    $addcell3keylabel = $insSubFr->Label(-text => $addCell3)->grid(-row => 2, -column => 0);
    $addcell3label = $insSubFr->Label(-text => "Add 1 $cell3label")->grid(-row => 2,
									  -column => 1,
									  -sticky => 'w');
    $subcell3keylabel = $insSubFr->Label(-text => $subCell3)->grid(-row => 2, -column => 3);
    $subcell3label = $insSubFr->Label(-text => "Subtract 1 $cell3label")->grid(-row => 2,
									       -column => 4,
									       -sticky => 'w');
    
    $addcell4keylabel = $insSubFr->Label(-text => $addCell4)->grid(-row => 3, -column => 0);
    $addcell4label = $insSubFr->Label(-text => "Add 1 $cell4label")->grid(-row => 3,
									  -column => 1,
									  -sticky => 'w');
    $subcell4keylabel = $insSubFr->Label(-text => $subCell4)->grid(-row => 3, -column => 3);
    $subcell4label = $insSubFr->Label(-text => "Subtract 1 $cell4label")->grid(-row => 3,
									       -column => 4,
									       -sticky => 'w');
    
    $addcell5keylabel = $insSubFr->Label(-text => $addCell5)->grid(-row => 4, -column => 0);
    $addcell5label = $insSubFr->Label(-text => "Add 1 $cell5label")->grid(-row => 4,
									  -column => 1,
									  -sticky => 'w');
    $subcell5keylabel = $insSubFr->Label(-text => $subCell5)->grid(-row => 4, -column => 3);
    $subcell5label = $insSubFr->Label(-text => "Subtract 1 $cell5label")->grid(-row => 4,
									       -column => 4,
									       -sticky => 'w');
    
    $addcell6keylabel = $insSubFr->Label(-text => $addCell6)->grid(-row => 5, -column => 0);
    $addcell6label = $insSubFr->Label(-text => "Add 1 $cell6label")->grid(-row => 5,
									  -column => 1,
									  -sticky => 'w');
    $subcell6keylabel = $insSubFr->Label(-text => $subCell6)->grid(-row => 5, -column => 3);
    $subcell6label = $insSubFr->Label(-text => "Subtract 1 $cell6label")->grid(-row => 5,
									       -column => 4,
									       -sticky => 'w');
    
    $addcell7keylabel = $insSubFr->Label(-text => $addCell7)->grid(-row => 6, -column => 0);
    $addcell7label = $insSubFr->Label(-text => "Add 1 $cell7label")->grid(-row => 6,
									  -column => 1,
									  -sticky => 'w');
    $subcell7keylabel = $insSubFr->Label(-text => $subCell7)->grid(-row => 6, -column => 3);
    $subcell7label = $insSubFr->Label(-text => "Subtract 1 $cell7label")->grid(-row => 6,
									       -column => 4,
									       -sticky => 'w');
    
# Counts Frame

    $cell1label_b = $countFr->Button(-text => $cell1label,
				     -font => $bigFont,
				     -command => sub { $cell1--;
						       $totalCellCount--;
						       &updateValues(@parameters);
						   },
				     -relief => 'raised',
				     -borderwidth => 5,
				     )->grid(-row => 0,
					     -column => 1,
					     -padx => 2,
					     -pady => 2,
					     );
    
    $cell1count = $countFr->NumEntryPlain(-value => $cell1,
					  -borderwidth => 0,
					  -width => 6,
					  -font => $bigFont,
					  -justify => 'center',
					  -state => 'readonly',
					  )->grid(-row => 0,
						  -column => 2,
						  -padx => 10,
						  );
    
    $cell2label_b = $countFr->Button(-text => $cell2label,
				     -font => $bigFont,
				     -command => sub { $cell2--;
						       $totalCellCount--;
						       &updateValues(@parameters);
						   },
				     -relief => 'raised',
				     -borderwidth => 5,
				     )->grid(-row => 1,
					     -column => 1,
					     -padx => 2,
					     -pady => 2,
					     );
    
    $cell2count = $countFr->NumEntryPlain(-value => $cell2,
					  -borderwidth => 0,
					  -state => 'readonly',
					  -width => 6,
					  -font => $bigFont,
					  -justify => 'center',
					  )->grid(-row => 1,
						  -column => 2,
						  );
    
    $cell3label_b = $countFr->Button(-text => $cell3label,
				     -font => $bigFont,
				     -command => sub { $cell3--;
						       $totalCellCount--;
						       &updateValues(@parameters);
						   },
				     -relief => 'raised',
				     -borderwidth => 5,
				     )->grid(-row => 2,
					     -column => 1,
					     -padx => 2,
					     -pady => 2,
					     );
    
    $cell3count = $countFr->NumEntryPlain(-value => $cell3,
					  -borderwidth => 0,
					  -state => 'readonly',
					  -width => 6,
					  -font => $bigFont,
					  -justify => 'center',
					  )->grid(-row => 2,
						  -column => 2,
						  );
    
    $cell4label_b = $countFr->Button(-text => $cell4label,
				     -font => $bigFont,
				     -command => sub { $cell4--;
						       $totalCellCount--;
						       &updateValues(@parameters);
						   },
				     -relief => 'raised',
				     -borderwidth => 5,
				     )->grid(-row => 3,
					     -column => 1,
					     -padx => 2,
					     -pady => 2,
					     );
    $cell4count = $countFr->NumEntryPlain(-value => $cell4,
					  -borderwidth => 0,
					  -state => 'readonly',
					  -width => 6,
					  -font => $bigFont,
					  -justify => 'center',
					  )->grid(-row => 3,
						  -column => 2,
						  );		     	 
    
    $cell5label_b = $countFr->Button(-text => $cell5label,
				     -font => $bigFont,
				     -command => sub { $cell5--;
						       $totalCellCount--;
						       &updateValues(@parameters);
						   },
				     -relief => 'raised',
				     -borderwidth => 5,
				     )->grid(-row => 4,
					     -column => 1,
					     -padx => 2,
					     -pady => 2,
					     );
    
    $cell5count = $countFr->NumEntryPlain(-value => $cell5,
					  -borderwidth => 0,
					  -state => 'readonly',
					  -width => 6,
					  -font => $bigFont,
					  -justify => 'center',
					  )->grid(-row => 4,
						  -column => 2,
						  );
    
    $cell6label_b = $countFr->Button(-text => $cell6label,
				     -font => $bigFont,
				     -command => sub { $cell6--;
						       $totalCellCount--;
						       &updateValues(@parameters);
						   },
				     -relief => 'raised',
				     -borderwidth => 5,
				     )->grid(-row => 0,
					     -column => 4,
					     -padx => 2,
					     -pady => 2,
					     );
    
    $cell6count = $countFr->NumEntryPlain(-value => $cell6,
					  -borderwidth => 0,
					  -state => 'readonly',
					  -width => 6,
					  -font => $bigFont,
					  -justify => 'center',
					  )->grid(-row => 0,
						  -column => 5,
						  );
    
    $cell7label_b = $countFr->Button(-text => $cell7label,
				     -font => $bigFont,
				     -command => sub { $cell7--;
						       $totalCellCount--;
						       &updateValues(@parameters);
						   },
				     -relief => 'raised',
				     -borderwidth => 5,
				     )->grid(-row => 1,
					     -column => 4,
					     -padx => 2,
					     -pady => 2,
					     );
    
    $cell7count = $countFr->NumEntryPlain(-value => $cell7,
					  -borderwidth => 0,
					  -state => 'readonly',
					  -width => 6,
					  -font => $bigFont,
					  -justify => 'center',
					  )->grid(-row => 1,
						  -column => 5,
						  );
    
    $countFr->Label(-text => 'Total Counts',
		    -font => $bigFont,
		    )->grid(-row => 2,
			    -column => 4);
    $totalCellCount_e = $countFr->NumEntryPlain(-value => $totalCellCount,
						-borderwidth => 0,
						-state => 'readonly',
						-width => 6,
						-font => $bigFont,
						-justify => 'center',
						)->grid(-row => 2,
							-column => 5,
							);
    
    $equation1label_e = $countFr->Label(-text => $equation1label,
					-font => $bigFont,
					)->grid(-row => 3,
						-column => 4);
    $equation1val_e = $countFr->NumEntryPlain(-value => 1,
					      -borderwidth => 0,
					      -state => 'readonly',
					      -width => 6,
					      -font => $bigFont,
					      -justify => 'center',
					      )->grid(-row => 3,
						      -column => 5,
						      );
    
    $equation2label_e = $countFr->Label(-text => $equation2label,
					-font => $bigFont,
					)->grid(-row => 4,
						-column => 4);
    $equation2val_e = $countFr->NumEntryPlain(-value => 1,
					      -borderwidth => 0,
					      -state => 'readonly',
					      -width => 6,
					      -font => $bigFont,
					      -justify => 'center',
					      )->grid(-row => 4,
						      -column => 5,
						      );
    
# Log Frame

    $logwin = $logFr->Scrolled("Text")->pack(-fill=>'both',
					     -expand=>1);
    $logwin->configure(-height=>5,
		       -width=>10,
		       -wrap=>'none',
		       -state=>'disabled',
		       );

# Keystroke Test Frame

    $keyTestWin = $keytestFr->Scrolled("Text")->pack(-fill=>'both',
						     -expand=>1);
    $keyTestWin->configure(-height=>27,
			   -width=>10,
			   -wrap=>'none',
			   -state=>'disabled',
			   );
    

# load & save settings file

    $optionFileFr->Button(-text => 'Load Settings File',
			  -font => $bigFont,
			  -borderwidth => 10,
			  -command => [ \&loadSettings, @parameters ],
			  )->grid(-row => 0,
				  -column => 1,
				  );
    $optionFileFr->Button(-text => 'Save Settings File',
			  -font => $bigFont,
			  -borderwidth => 10,
			  -command => [ \&saveSettings, @parameters ],
			  )->grid(-row => 0,
				  -column => 3,
				  );

    # spacer
    $optionFileFr->Label->grid(-row => 0,
			       -column => 2,
			       -padx => 50,
			       );


# Command Frame
    $commandFrBot->Button(-text => 'Reset',
			  -font => $bigFont,
			  -width => 6,
			  -height => 2,
			  -borderwidth => 10,
			  -command => sub { $cell1 = 0;
					    $cell2 = 0;
					    $cell3 = 0;
					    $cell4 = 0;
					    $cell5 = 0;
					    $cell6 = 0;
					    $cell7 = 0;
					    $totalCellCount = 0;
					    &updateValues(@parameters);
					    
					    $logwin->configure(-state => 'normal');
					    $logwin->delete('0.0','end');
					    $logwin->configure(-state => 'disabled');
					    $logwin->update;
					    
					    $keyTestWin->configure(-state => 'normal');
					    $keyTestWin->delete('0.0','end');
					    $keyTestWin->configure(-state => 'disabled');
					    $keyTestWin->update;
					    
					},
			  )->grid(-row => 0,
				  -column => 0,
				  -padx => 20,
				  -pady => 20,
				  );
    
    $commandFrTop->Button(-text => 'Exit',
			  -font => $bigFont,
			  -height => 2,
			  -width => 6,
			  -borderwidth => 10,
			  -command => sub { &autoSaveOnExit(@parameters);
					    exit;
					},
			  )->grid(-row => 0,
				  -column => 1,
				  -padx => 20,
				  -pady => 20,
				  );

# Help

    my $helpTextBox = $helpTab->Scrolled('ROText',
                                         -background => 'white',
                                         -wrap => 'word',
                                         -scrollbars => 'osoe',
                                         )->pack(-fill => 'both',
                                                 -expand => 1,
                                                 );
    
    my $helpText = <<EOF;
=== About ===

Copyright Charlie Kim 2007-2009

Send questions, comments, and features requests to cckim47\@gmail.com

This program is free software: you can redistribute it and/or modify it under the terms of the GNU General Public License as publisehd by the Free Software Foundation, either version 3 of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License at http://www.gnu.org/licenses/ for more details.


=== System Requirements ===

The software was developed using ActiveState Perl running on Windows XP.  Hardware requirements are minimal; it has been confirmed to run well on a system with a 500 MHz CPU.

Running directly from source will require Perl and a number of modules which are available through CPAN.


=== Usage: Overview ===

VersaCount was designed with the goal of full configurability in the context of a manual multi-unit counter.  The software is designed to allow the user to specify many options to customize input and output interfaces.  Once these parameters have been configured, the settings can be saved in a file and reloaded as desired.  This permits rapid reconfiguration of all settings, allowing the interface to be instantly customized for different applications.

One key feature which distinguishes VersaCount from standard hand tally counters is the ability to simultaneously count multiple events with a single keystroke, which results in significant reductions in the time spent counting.  See "Usage: Multi-counting" for details.


=== Usage: Counting ===

VersaCount is designed to function similarly to a standard multi-unit handy tally counter of the type commonly found at cell counting stations in laboratories.  The similarity in use, with shortcut keys substituting for the levers of manual counters, allows for rapid adoption of the software for the replacement of manual counters.

In addition, a running history of keystrokes is available on the main "Counter" tab for review.  In the case of keystroke errors, events can be subtracted either by using the user-specified keystroke shortcut or by clicking on the name of the category to be subtracted.  This feature is particularly useful in configurations which utilize just a touchscreen monitor and numeric keypad in the absence of a full keyboard and mouse.


=== Usage: Multi-counting ===

The "Cell 1" register is specially reserved as a category which accepts multiple addition inputs for 1 to 9 simultaneous events.  The human brain can process visual fields significantly more rapidly than the hand can repeatedly press a lever or keystroke.  By using single keystrokes in place of multiple presses of a lever on a traditional tally counter, substantially decreased counting times can be achieved.  Although this new approach to counting requires practice, the improved performance will be quickly realized.

The ability to simultaneously add 1 to 9 events was designed to correspond to the 1 to 9 keys of the numeric keypad.  However, these events can be configured to any desired keystrokes using the procedures described in the next section.


=== Usage: Configuring the keys ===

Different keyboards and keypads send the same keystrokes under different codes, so it is important to configure your VersaCount settings to capture the exact codes sent by your input device.  Note that this does not necessarily reflect the text shown on the key itself.

After starting VersaCount, click on the "Keystroke Test" tab.  Here, you can experiment with different key presses to identify how your input device's keystrokes are received by the VersaCount software.  Write these values down and switch back to the "Options" tab and type the values into their corresponding entry boxes.  Note again that simply pressing the key may not equate to how the keystroke is logged, and entire words may be required to receive the keystrokes correctly.  Some common numeric keypad examples are: Insert, comma, Delete, Return, plus, minus, End, Down, Next, Left, Win_L, Right, home Up, and Prior.  Note that the names are case-sensitive!


=== Usage: Configuring equations ===

Equations can be used to track multi-component parameters.  For example, one may wish to know a specific percentage of the total events constituted by a single or subset of categories (e.g. "Live cells").  

In the current configuration, two equations can be specified under the "Options" tab.  The syntax used for the equations is the same as that used by Perl.  The seven cell categories are referred to by their variable names - "\$cell1" through "\$cell7" (without the quotation marks).  Standard mathematical operations are possible, and parentheses are recognized.  In theory, any mathematical function allowed in Perl can be used as long as Perl's syntax is observed, although this has not been fully tested.

An example that one might use to keep track of the percentage proportion of "Cell 2" and "Cell 3" in a 5 category count total would be written as follows:

100 * (\$cell2 + \$cell3) / (\$cell1 + \$cell2 + \$cell3 + \$cell4 + \$cell5)

The equations can be given names under the "Options" tab that will show up on the "Counter" tab (e.g. "Live cells").


=== Usage: Configuring sounds and alarms ===

The use of different sounds for different cell categories gives audible feedback that the correct keystroke was made, which is useful if your eyes are unable to monitor the screen (e.g. when looking through a microscope while counting cells).  The audio files must be in the .wav format.  Because playing audio is blocking (i.e. counting cannot be done while the audio plays), it is generally a good rule to use only short audio samples such as the beeps provided.

Another useful type of sound is alarms.  These are additional sounds which are triggered upon reaching a user-specified number of cells in the "Options" tab.  When these thresholds are crossed in the "Total Cells" field, the corresponding sound file is played.  This is useful if a pre-determined number of cells is to be counted.

EOF

    $helpTextBox->insert('end',$helpText);




# The Action
    
    $mw->bind( '<Any-KeyPress>' => [ \&readBindings, @parameters ] );
    
    &updateValues(@parameters);

    MainLoop();
}


sub readBindings {
    my $c = shift; # the keypress
    my @parameters = @_;
    my ($mwref,
	$cell1countref,
	$cell2countref,
	$cell3countref,
	$cell4countref,
	$cell5countref,
	$cell6countref,
	$cell7countref,
	$totalCellCountref,
	$totalCellCount_eref,
	$equation1ref,
	$equation2ref,
	$equation1labelref,
	$equation2labelref,
	$equation1val_eref,
	$equation2val_eref,
	$cell1ref,
	$cell2ref,
	$cell3ref,
	$cell4ref,
	$cell5ref,
	$cell6ref,
	$cell7ref,
	$cell1labelref,
	$cell2labelref,
	$cell3labelref,
	$cell4labelref,
	$cell5labelref,
	$cell6labelref,
	$cell7labelref,
	$cell1label_bref,
	$cell2label_bref,
	$cell3label_bref,
	$cell4label_bref,
	$cell5label_bref,
	$cell6label_bref,
	$cell7label_bref,
	$logwinref,
	$bookref,
	$keyTestWinref,
	$addCell2ref,
	$addCell3ref,
	$addCell4ref,
	$addCell5ref,
	$add1cell1ref,
	$add2cell1ref,
	$add3cell1ref,
	$add4cell1ref,
	$add5cell1ref,
	$add6cell1ref,
	$add7cell1ref,
	$add8cell1ref,
	$add9cell1ref,
	$addCell6ref,
	$addCell7ref,
	$subCell1ref,
	$subCell2ref,
	$subCell3ref,
	$subCell4ref,
	$subCell5ref,
	$subCell6ref,
	$subCell7ref,
	$soundToggleref,
	$soundCell1ref,
	$soundCell2ref,
	$soundCell3ref,
	$soundCell4ref,
	$soundCell5ref,
	$soundCell6ref,
	$soundCell7ref,
	$soundAlert1ref,
	$soundAlert2ref,
	$soundAlert1valref,
	$soundAlert2valref,
	$audioDataCell1ref,
	$audioDataCell2ref,
	$audioDataCell3ref,
	$audioDataCell4ref,
	$audioDataCell5ref,
	$audioDataCell6ref,
	$audioDataCell7ref,
	$audioDataAlert1ref,
	$audioDataAlert2ref,
	$svrref,
	) = @parameters;
    my $e = $c->XEvent;
    my( $k, $K, $W, $A ) = ( $e->k, $e->K, $e->W, $e->A );

    my $focusTab = $$bookref->info("focus");
#    print "Keyed $k in tab $focusTab\n";
#    print "Keyed $K in tab $focusTab\n";

    if ($focusTab eq 'Keystroke Test') {
	&printlog("$K\n",$keyTestWinref);
    }

    elsif ($focusTab eq 'Options') {
	return;
    }

    elsif ($focusTab eq 'Counter') {

	my $lastTotal = $$totalCellCountref;

#	$$mwref->focus;

	# OS check
	my $os = 'other';
	if ($^O =~ /MSWin32/i) {
	    $os = 'win';
	}

	# add cell2
	if ($K eq $$addCell2ref) {
	    $$cell2ref++;
	    $$totalCellCountref++;
	    
	    Win32::Sound::Play($$soundCell2ref) if ($os eq 'win' && $$soundToggleref);
#	    $$svrref->play($$audioDataCell2ref) if ($os eq 'other' && $$soundToggleref);
	    

	    if ( $lastTotal % $$soundAlert2valref > $$totalCellCountref % $$soundAlert2valref ) {
		Win32::Sound::Play($$soundAlert2ref) if ($os eq 'win' && $$soundToggleref);
#		    $$svrref->play($$audioData100ref) if ($os eq 'other' && $$soundToggleref);
#		select(undef, undef, undef, 0.1);
	      }
	    
	    if ( $lastTotal % $$soundAlert1valref > $$totalCellCountref % $$soundAlert1valref ) {
		Win32::Sound::Play($$soundAlert1ref) if ($os eq 'win' && $$soundToggleref);
#		    $$svrref->play($$audioData100ref) if ($os eq 'other' && $$soundToggleref);
#		select(undef, undef, undef, 0.1);
	      }
	    &updateValues(@parameters);
	    &printlog("$$addCell2ref\n",$logwinref);
	}
	
	# add cell3
	elsif ($K eq $$addCell3ref) {
	    $$cell3ref++;
	    $$totalCellCountref++;

	    Win32::Sound::Play($$soundCell3ref) if ($os eq 'win' && $$soundToggleref);
#	    $$svrref->play($$audioDataCell3ref) if ($os eq 'other' && $$soundToggleref);

	    if ( $lastTotal % $$soundAlert2valref > $$totalCellCountref % $$soundAlert2valref ) {
		Win32::Sound::Play($$soundAlert2ref) if ($os eq 'win' && $$soundToggleref);
#		    $$svrref->play($$audioData100ref) if ($os eq 'other' && $$soundToggleref);
#		select(undef, undef, undef, 0.1);
	      }
	    
	    if ( $lastTotal % $$soundAlert1valref > $$totalCellCountref % $$soundAlert1valref ) {
		Win32::Sound::Play($$soundAlert1ref) if ($os eq 'win' && $$soundToggleref);
#		    $$svrref->play($$audioData100ref) if ($os eq 'other' && $$soundToggleref);
#		select(undef, undef, undef, 0.1);
	      }
	    &updateValues(@parameters);
	    &printlog("$$addCell3ref\n",$logwinref);
	}
	
	# add cell4
	elsif ($K eq $$addCell4ref) {
	    $$cell4ref++;
	    $$totalCellCountref++;

	    Win32::Sound::Play($$soundCell4ref) if ($os eq 'win' && $$soundToggleref);
#	    $$svrref->play($$audioDataCell4ref) if ($os eq 'other' && $$soundToggleref);

	    if ( $lastTotal % $$soundAlert2valref > $$totalCellCountref % $$soundAlert2valref ) {
		Win32::Sound::Play($$soundAlert2ref) if ($os eq 'win' && $$soundToggleref);
#		    $$svrref->play($$audioData100ref) if ($os eq 'other' && $$soundToggleref);
#		select(undef, undef, undef, 0.1);
	      }
	    
	    if ( $lastTotal % $$soundAlert1valref > $$totalCellCountref % $$soundAlert1valref ) {
		Win32::Sound::Play($$soundAlert1ref) if ($os eq 'win' && $$soundToggleref);
#		    $$svrref->play($$audioData100ref) if ($os eq 'other' && $$soundToggleref);
#		select(undef, undef, undef, 0.1);
	      }
	    &updateValues(@parameters);
	    &printlog("$$addCell4ref\n",$logwinref);
	}
	
	# add cell5
	elsif ($K eq $$addCell5ref) {
	    $$cell5ref++;
	    $$totalCellCountref++;

	    Win32::Sound::Play($$soundCell5ref) if $$soundToggleref;
# no svrref?

	    if ( $lastTotal % $$soundAlert2valref > $$totalCellCountref % $$soundAlert2valref ) {
		Win32::Sound::Play($$soundAlert2ref) if ($os eq 'win' && $$soundToggleref);
#		    $$svrref->play($$audioData100ref) if ($os eq 'other' && $$soundToggleref);
#		select(undef, undef, undef, 0.1);
	      }
	    
	    if ( $lastTotal % $$soundAlert1valref > $$totalCellCountref % $$soundAlert1valref ) {
		Win32::Sound::Play($$soundAlert1ref) if ($os eq 'win' && $$soundToggleref);
#		    $$svrref->play($$audioData100ref) if ($os eq 'other' && $$soundToggleref);
#		select(undef, undef, undef, 0.1);
	      }
	    &updateValues(@parameters);
	    &printlog("$$addCell5ref\n",$logwinref);
	}
	
	# add cell6
	elsif ($K eq $$addCell6ref) {
	    $$cell6ref++;
	    $$totalCellCountref++;

	    Win32::Sound::Play($$soundCell6ref) if $$soundToggleref;

	    if ( $lastTotal % $$soundAlert2valref > $$totalCellCountref % $$soundAlert2valref ) {
		Win32::Sound::Play($$soundAlert2ref) if ($os eq 'win' && $$soundToggleref);
#		    $$svrref->play($$audioData100ref) if ($os eq 'other' && $$soundToggleref);
#		select(undef, undef, undef, 0.1);
	      }
	    
	    if ( $lastTotal % $$soundAlert1valref > $$totalCellCountref % $$soundAlert1valref ) {
		Win32::Sound::Play($$soundAlert1ref) if ($os eq 'win' && $$soundToggleref);
#		    $$svrref->play($$audioData100ref) if ($os eq 'other' && $$soundToggleref);
#		select(undef, undef, undef, 0.1);
	      }

	    &updateValues(@parameters);
	    &printlog("$$addCell6ref\n",$logwinref);
	}
	
	# add cell7
	elsif ($K eq $$addCell7ref) {
	    $$cell7ref++;
	    $$totalCellCountref++;
	    Win32::Sound::Play($$soundCell7ref) if $$soundToggleref;

	    if ( $lastTotal % $$soundAlert2valref > $$totalCellCountref % $$soundAlert2valref ) {
		Win32::Sound::Play($$soundAlert2ref) if ($os eq 'win' && $$soundToggleref);
#		    $$svrref->play($$audioData100ref) if ($os eq 'other' && $$soundToggleref);
#		select(undef, undef, undef, 0.1);
	      }
	    
	    if ( $lastTotal % $$soundAlert1valref > $$totalCellCountref % $$soundAlert1valref ) {
		Win32::Sound::Play($$soundAlert1ref) if ($os eq 'win' && $$soundToggleref);
#		    $$svrref->play($$audioData100ref) if ($os eq 'other' && $$soundToggleref);
#		select(undef, undef, undef, 0.1);
	      }

	    &updateValues(@parameters);
	    &printlog("$$addCell7ref\n",$logwinref);
	}
	
	# add 1 cell1
	elsif ($K eq $$add1cell1ref) {
	    my $value = 1;
	    $$cell1ref += $value;
	    $$totalCellCountref += $value;

	    Win32::Sound::Play($$soundCell1ref) if $$soundToggleref;

	    if ( $lastTotal % $$soundAlert2valref > $$totalCellCountref % $$soundAlert2valref ) {
		Win32::Sound::Play($$soundAlert2ref) if ($os eq 'win' && $$soundToggleref);
#		    $$svrref->play($$audioData100ref) if ($os eq 'other' && $$soundToggleref);
#		select(undef, undef, undef, 0.1);
	      }
	    
	    if ( $lastTotal % $$soundAlert1valref > $$totalCellCountref % $$soundAlert1valref ) {
		Win32::Sound::Play($$soundAlert1ref) if ($os eq 'win' && $$soundToggleref);
#		    $$svrref->play($$audioData100ref) if ($os eq 'other' && $$soundToggleref);
#		select(undef, undef, undef, 0.1);
	      }
	    &updateValues(@parameters);
	    &printlog("$value\n",$logwinref);
	}

	# add 2 cell1
	elsif ($K eq $$add2cell1ref) {
	    my $value = 2;
	    $$cell1ref += $value;
	    $$totalCellCountref += $value;

	    Win32::Sound::Play($$soundCell1ref) if $$soundToggleref;

	    if ( $lastTotal % $$soundAlert2valref > $$totalCellCountref % $$soundAlert2valref ) {
		Win32::Sound::Play($$soundAlert2ref) if ($os eq 'win' && $$soundToggleref);
#		    $$svrref->play($$audioData100ref) if ($os eq 'other' && $$soundToggleref);
#		select(undef, undef, undef, 0.1);
	      }
	    
	    if ( $lastTotal % $$soundAlert1valref > $$totalCellCountref % $$soundAlert1valref ) {
		Win32::Sound::Play($$soundAlert1ref) if ($os eq 'win' && $$soundToggleref);
#		    $$svrref->play($$audioData100ref) if ($os eq 'other' && $$soundToggleref);
#		select(undef, undef, undef, 0.1);
	      }
	    &updateValues(@parameters);
	    &printlog("$value\n",$logwinref);
	}

	# add 3 cell1
	elsif ($K eq $$add3cell1ref) {
	    my $value = 3;
	    $$cell1ref += $value;
	    $$totalCellCountref += $value;

	    Win32::Sound::Play($$soundCell1ref) if $$soundToggleref;

	    if ( $lastTotal % $$soundAlert2valref > $$totalCellCountref % $$soundAlert2valref ) {
		Win32::Sound::Play($$soundAlert2ref) if ($os eq 'win' && $$soundToggleref);
#		    $$svrref->play($$audioData100ref) if ($os eq 'other' && $$soundToggleref);
#		select(undef, undef, undef, 0.1);
	      }
	    
	    if ( $lastTotal % $$soundAlert1valref > $$totalCellCountref % $$soundAlert1valref ) {
		Win32::Sound::Play($$soundAlert1ref) if ($os eq 'win' && $$soundToggleref);
#		    $$svrref->play($$audioData100ref) if ($os eq 'other' && $$soundToggleref);
#		select(undef, undef, undef, 0.1);
	      }
	    &updateValues(@parameters);
	    &printlog("$value\n",$logwinref);
	}

	# add 4 cell1
	elsif ($K eq $$add4cell1ref) {
	    my $value = 4;
	    $$cell1ref += $value;
	    $$totalCellCountref += $value;

	    Win32::Sound::Play($$soundCell1ref) if $$soundToggleref;

	    if ( $lastTotal % $$soundAlert2valref > $$totalCellCountref % $$soundAlert2valref ) {
		Win32::Sound::Play($$soundAlert2ref) if ($os eq 'win' && $$soundToggleref);
#		    $$svrref->play($$audioData100ref) if ($os eq 'other' && $$soundToggleref);
#		select(undef, undef, undef, 0.1);
	      }
	    
	    if ( $lastTotal % $$soundAlert1valref > $$totalCellCountref % $$soundAlert1valref ) {
		Win32::Sound::Play($$soundAlert1ref) if ($os eq 'win' && $$soundToggleref);
#		    $$svrref->play($$audioData100ref) if ($os eq 'other' && $$soundToggleref);
#		select(undef, undef, undef, 0.1);
	      }
	    &updateValues(@parameters);
	    &printlog("$value\n",$logwinref);
	}

	# add 5 cell1
	elsif ($K eq $$add5cell1ref) {
	    my $value = 5;
	    $$cell1ref += $value;
	    $$totalCellCountref += $value;

	    Win32::Sound::Play($$soundCell1ref) if $$soundToggleref;

	    if ( $lastTotal % $$soundAlert2valref > $$totalCellCountref % $$soundAlert2valref ) {
		Win32::Sound::Play($$soundAlert2ref) if ($os eq 'win' && $$soundToggleref);
#		    $$svrref->play($$audioData100ref) if ($os eq 'other' && $$soundToggleref);
#		select(undef, undef, undef, 0.1);
	      }
	    
	    if ( $lastTotal % $$soundAlert1valref > $$totalCellCountref % $$soundAlert1valref ) {
		Win32::Sound::Play($$soundAlert1ref) if ($os eq 'win' && $$soundToggleref);
#		    $$svrref->play($$audioData100ref) if ($os eq 'other' && $$soundToggleref);
#		select(undef, undef, undef, 0.1);
	      }
	    &updateValues(@parameters);
	    &printlog("$value\n",$logwinref);
	}

	# add 6 cell1
	elsif ($K eq $$add6cell1ref) {
	    my $value = 6;
	    $$cell1ref += $value;
	    $$totalCellCountref += $value;

	    Win32::Sound::Play($$soundCell1ref) if $$soundToggleref;

	    if ( $lastTotal % $$soundAlert2valref > $$totalCellCountref % $$soundAlert2valref ) {
		Win32::Sound::Play($$soundAlert2ref) if ($os eq 'win' && $$soundToggleref);
#		    $$svrref->play($$audioData100ref) if ($os eq 'other' && $$soundToggleref);
#		select(undef, undef, undef, 0.1);
	      }
	    
	    if ( $lastTotal % $$soundAlert1valref > $$totalCellCountref % $$soundAlert1valref ) {
		Win32::Sound::Play($$soundAlert1ref) if ($os eq 'win' && $$soundToggleref);
#		    $$svrref->play($$audioData100ref) if ($os eq 'other' && $$soundToggleref);
#		select(undef, undef, undef, 0.1);
	      }
	    &updateValues(@parameters);
	    &printlog("$value\n",$logwinref);
	}

	# add 7 cell1
	elsif ($K eq $$add7cell1ref) {
	    my $value = 7;
	    $$cell1ref += $value;
	    $$totalCellCountref += $value;

	    Win32::Sound::Play($$soundCell1ref) if $$soundToggleref;

	    if ( $lastTotal % $$soundAlert2valref > $$totalCellCountref % $$soundAlert2valref ) {
		Win32::Sound::Play($$soundAlert2ref) if ($os eq 'win' && $$soundToggleref);
#		    $$svrref->play($$audioData100ref) if ($os eq 'other' && $$soundToggleref);
#		select(undef, undef, undef, 0.1);
	      }
	    
	    if ( $lastTotal % $$soundAlert1valref > $$totalCellCountref % $$soundAlert1valref ) {
		Win32::Sound::Play($$soundAlert1ref) if ($os eq 'win' && $$soundToggleref);
#		    $$svrref->play($$audioData100ref) if ($os eq 'other' && $$soundToggleref);
#		select(undef, undef, undef, 0.1);
	      }
	    &updateValues(@parameters);
	    &printlog("$value\n",$logwinref);
	}

	# add 8 cell1
	elsif ($K eq $$add8cell1ref) {
	    my $value = 8;
	    $$cell1ref += $value;
	    $$totalCellCountref += $value;

	    Win32::Sound::Play($$soundCell1ref) if $$soundToggleref;

	    if ( $lastTotal % $$soundAlert2valref > $$totalCellCountref % $$soundAlert2valref ) {
		Win32::Sound::Play($$soundAlert2ref) if ($os eq 'win' && $$soundToggleref);
#		    $$svrref->play($$audioData100ref) if ($os eq 'other' && $$soundToggleref);
#		select(undef, undef, undef, 0.1);
	      }
	    
	    if ( $lastTotal % $$soundAlert1valref > $$totalCellCountref % $$soundAlert1valref ) {
		Win32::Sound::Play($$soundAlert1ref) if ($os eq 'win' && $$soundToggleref);
#		    $$svrref->play($$audioData100ref) if ($os eq 'other' && $$soundToggleref);
#		select(undef, undef, undef, 0.1);
	      }
	    &updateValues(@parameters);
	    &printlog("$value\n",$logwinref);
	}

	# add 9 cell1
	elsif ($K eq $$add9cell1ref) {
	    my $value = 9;
	    $$cell1ref += $value;
	    $$totalCellCountref += $value;

	    Win32::Sound::Play($$soundCell1ref) if $$soundToggleref;

	    if ( $lastTotal % $$soundAlert2valref > $$totalCellCountref % $$soundAlert2valref ) {
		Win32::Sound::Play($$soundAlert2ref) if ($os eq 'win' && $$soundToggleref);
#		    $$svrref->play($$audioData100ref) if ($os eq 'other' && $$soundToggleref);
#		select(undef, undef, undef, 0.1);
	      }
	    
	    if ( $lastTotal % $$soundAlert1valref > $$totalCellCountref % $$soundAlert1valref ) {
		Win32::Sound::Play($$soundAlert1ref) if ($os eq 'win' && $$soundToggleref);
#		    $$svrref->play($$audioData100ref) if ($os eq 'other' && $$soundToggleref);
#		select(undef, undef, undef, 0.1);
	      }
	    &updateValues(@parameters);
	    &printlog("$value\n",$logwinref);
	}

	# subtract cell1
	elsif ($K eq $$subCell1ref) {
	    $$cell1ref--;
	    $$totalCellCountref--;
	    &updateValues(@parameters);
	    &printlog("$$subCell1ref\n",$logwinref);
	}
	
	# subtract cell2
	elsif ($K eq $$subCell2ref) {
	    my $value = $1;
	    $$cell2ref--;
	    $$totalCellCountref--;
	    &updateValues(@parameters);
	    &printlog("$$subCell2ref\n",$logwinref);
	}
	
	# subtract cell3
	elsif ($K eq $$subCell3ref) {
	    my $value = $1;
	    $$cell3ref--;
	    $$totalCellCountref--;
	    &updateValues(@parameters);
	    &printlog("$$subCell3ref\n",$logwinref);
	}
	
	# subtract cell4
	elsif ($K eq $$subCell4ref) {
	    my $value = $1;
	    $$cell4ref--;
	    $$totalCellCountref--;
	    &updateValues(@parameters);
	    &printlog("$$subCell4ref\n",$logwinref);
	}
	
	# subtract cell5
	elsif ($K eq $$subCell5ref) {
	    my $value = $1;
	    $$cell5ref--;
	    $$totalCellCountref--;
	    &updateValues(@parameters);
	    &printlog("$$subCell5ref\n",$logwinref);
	}
	
	# subtract cell6
	elsif ($K eq $$subCell6ref) {
	    my $value = $1;
	    $$cell6ref--;
	    $$totalCellCountref--;
	    &updateValues(@parameters);
	    &printlog("$$subCell6ref\n",$logwinref);
	}
	
	# subtract cell7
	elsif ($K eq $$subCell7ref) {
	    my $value = $1;
	    $$cell7ref--;
	    $$totalCellCountref--;
	    &updateValues(@parameters);
	    &printlog("$$subCell7ref\n",$logwinref);
	}
	
	else {
	    return;
	}
    }
}

sub printlog {
    my $line = shift;
    my $logwinref = shift;
    my $logwin = $$logwinref;
    $logwin->configure(-state=>'normal');
    $logwin->insert('end',$line);
    $logwin->yview(moveto=>1);
    $logwin->configure(-state=>'disabled');
    $logwin->update();
}

sub calcParasitemia {
    my ($totalParref, $totalCellCountref) = @_;
    if ($$totalCellCountref == 0) {
	return 0;
    }
    else {
	return $$totalParref / $$totalCellCountref;
    }
}

sub updateValues {
    my ($mwref,
	$cell1countref,
	$cell2countref,
	$cell3countref,
	$cell4countref,
	$cell5countref,
	$cell6countref,
	$cell7countref,
	$totalCellCountref,
	$totalCellCount_eref,
	$equation1ref,
	$equation2ref,
	$equation1labelref,
	$equation2labelref,
	$equation1val_eref,
	$equation2val_eref,
	$cell1ref,
	$cell2ref,
	$cell3ref,
	$cell4ref,
	$cell5ref,
	$cell6ref,
	$cell7ref,
	$cell1labelref,
	$cell2labelref,
	$cell3labelref,
	$cell4labelref,
	$cell5labelref,
	$cell6labelref,
	$cell7labelref,
	$cell1label_bref,
	$cell2label_bref,
	$cell3label_bref,
	$cell4label_bref,
	$cell5label_bref,
	$cell6label_bref,
	$cell7label_bref,
	$logwinref,
	$bookref,
	$keyTestWinref,
	$addCell2ref,
	$addCell3ref,
	$addCell4ref,
	$addCell5ref,
	$add1cell1ref,
	$add2cell1ref,
	$add3cell1ref,
	$add4cell1ref,
	$add5cell1ref,
	$add6cell1ref,
	$add7cell1ref,
	$add8cell1ref,
	$add9cell1ref,
	$addCell6ref,
	$addCell7ref,
	$subCell1ref,
	$subCell2ref,
	$subCell3ref,
	$subCell4ref,
	$subCell5ref,
	$subCell6ref,
	$subCell7ref,
	$soundToggleref,
	$soundCell1ref,
	$soundCell2ref,
	$soundCell3ref,
	$soundCell4ref,
	$soundCell5ref,
	$soundCell6ref,
	$soundCell7ref,
	$soundAlert1ref,
	$soundAlert2ref,
	$soundAlert1valref,
	$soundAlert2valref,
	$audioDataCell1ref,
	$audioDataCell2ref,
	$audioDataCell3ref,
	$audioDataCell4ref,
	$audioDataCell5ref,
	$audioDataCell6ref,
	$audioDataCell7ref,
	$audioDataAlert1ref,
	$audioDataAlert2ref,
	$svrref,
	) = @_;

    $$cell1countref->configure(-state => 'normal');
    $$cell1countref->configure(-value => $$cell1ref);
    $$cell1countref->configure(-state => 'readonly');
    $$cell1countref->update;
    
    $$cell2countref->configure(-state => 'normal');
    $$cell2countref->configure(-value => $$cell2ref);
    $$cell2countref->configure(-state => 'readonly');
    $$cell2countref->update;
    
    $$cell3countref->configure(-state => 'normal');
    $$cell3countref->configure(-value => $$cell3ref);
    $$cell3countref->configure(-state => 'readonly');
    $$cell3countref->update;

    $$cell4countref->configure(-state => 'normal');
    $$cell4countref->configure(-value => $$cell4ref);
    $$cell4countref->configure(-state => 'readonly');
    $$cell4countref->update;

    $$cell5countref->configure(-state => 'normal');
    $$cell5countref->configure(-value => $$cell5ref);
    $$cell5countref->configure(-state => 'readonly');
    $$cell5countref->update;

    $$cell6countref->configure(-state => 'normal');
    $$cell6countref->configure(-value => $$cell6ref);
    $$cell6countref->configure(-state => 'readonly');
    $$cell6countref->update;

    $$cell7countref->configure(-state => 'normal');
    $$cell7countref->configure(-value => $$cell7ref);
    $$cell7countref->configure(-state => 'readonly');
    $$cell7countref->update;

    $$totalCellCount_eref->configure(-state => 'normal');
    $$totalCellCount_eref->configure(-value => $$totalCellCountref);
    $$totalCellCount_eref->configure(-state => 'readonly');
    $$totalCellCount_eref->update;

    my $cell1 = $$cell1ref;
    my $cell2 = $$cell2ref;
    my $cell3 = $$cell3ref;
    my $cell4 = $$cell4ref;
    my $cell5 = $$cell5ref;
    my $cell6 = $$cell6ref;
    my $cell7 = $$cell7ref;


    # NEED TO TEST FOR DIVISION BY ZERO
    my $eq1val = eval($$equation1ref);
    if ($eq1val) {
	$eq1val = sprintf("%.5s",$eq1val);
    }
    else {
	$eq1val = 0;
    }
    $$equation1val_eref->configure(-state => 'normal');
    $$equation1val_eref->configure(-value => $eq1val);
    $$equation1val_eref->configure(-state => 'readonly');
    $$equation1val_eref->update;

    my $eq2val = eval($$equation2ref);
    if ($eq2val) {
	$eq2val = sprintf("%.5s",$eq2val) if $eq2val;
    }
    else {
	$eq2val = 0;
    }
    $$equation2val_eref->configure(-state => 'normal');
    $$equation2val_eref->configure(-value => $eq2val);
    $$equation2val_eref->configure(-state => 'readonly');
    $$equation2val_eref->update;
}

sub loadSettings {
    my ($mwref,
	$cell1countref,
	$cell2Countref,
	$cell3Countref,
	$cell4Countref,
	$cell5Countref,
	$cell6countref,
	$cell7countref,
	$totalCellCountref,
	$totalCellCount_eref,
	$equation1ref,
	$equation2ref,
	$equation1labelref,
	$equation2labelref,
	$equation1val_eref,
	$equation2val_eref,
	$cell1ref,
	$cell2ref,
	$cell3ref,
	$cell4ref,
	$cell5ref,
	$cell6ref,
	$cell7ref,
	$cell1labelref,
	$cell2labelref,
	$cell3labelref,
	$cell4labelref,
	$cell5labelref,
	$cell6labelref,
	$cell7labelref,
	$cell1label_bref,
	$cell2label_bref,
	$cell3label_bref,
	$cell4label_bref,
	$cell5label_bref,
	$cell6label_bref,
	$cell7label_bref,
	$logwinref,
	$bookref,
	$keyTestWinref,
	$addCell2ref,
	$addCell3ref,
	$addCell4ref,
	$addCell5ref,
	$add1cell1ref,
	$add2cell1ref,
	$add3cell1ref,
	$add4cell1ref,
	$add5cell1ref,
	$add6cell1ref,
	$add7cell1ref,
	$add8cell1ref,
	$add9cell1ref,
	$addCell6ref,
	$addCell7ref,
	$subCell1ref,
	$subCell2ref,
	$subCell3ref,
	$subCell4ref,
	$subCell5ref,
	$subCell6ref,
	$subCell7ref,
	$soundToggleref,
	$soundCell1ref,
	$soundCell2ref,
	$soundCell3ref,
	$soundCell4ref,
	$soundCell5ref,
	$soundCell6ref,
	$soundCell7ref,
	$soundAlert1ref,
	$soundAlert2ref,
	$soundAlert1valref,
	$soundAlert2valref,
	$audioDataCell1ref,
	$audioDataCell2ref,
	$audioDataCell3ref,
	$audioDataCell4ref,
	$audioDataCell5ref,
	$audioDataCell6ref,
	$audioDataCell7ref,
	$audioDataAlert1ref,
	$audioDataAlert2ref,
	$svrref,
	) = @_;

    my $file;
    if ($^O =~ /mswin/i) {
	my $opentypes = [
			 "{All files}     *           ",
			 "{Text files}    {.txt}      ",
			 ];
	$file = $$mwref->getOpenFile(-filetypes => $opentypes,
				     -title => 'Load Settings',
				     );
    }
    else {
	$file = $$mwref->getOpenFile(-title => 'Load File');
    }
    if (defined $file and $file ne '') {
	# read settings from file
	open(LOAD,$file) or die "Couldn't open file $file: $!\n";
	my %settings;
	while(<LOAD>) {
	    chomp;
	    $_ =~ s/\r//g;
	    my @line = split(/\t/);
	    $settings{$line[0]} = $line[1];
	}
	close LOAD;
	
	# assign settings to variables

	$$cell1labelref = $settings{'Cell1label'};
	$$cell2labelref = $settings{'Cell2label'};
	$$cell3labelref = $settings{'Cell3label'};
	$$cell4labelref = $settings{'Cell4label'};
	$$cell5labelref = $settings{'Cell5label'};
	$$cell6labelref = $settings{'Cell6label'};
	$$cell7labelref = $settings{'Cell7label'};

	$$addCell2ref = $settings{'AddCell2'};
	$$addCell3ref = $settings{'AddCell3'};
	$$addCell4ref = $settings{'AddCell4'};
	$$addCell5ref = $settings{'AddCell5'};
	$$add1cell1ref = $settings{'Add1cell1'};
	$$add2cell1ref = $settings{'Add2cell1'};
	$$add3cell1ref = $settings{'Add3cell1'};
	$$add4cell1ref = $settings{'Add4cell1'};
	$$add5cell1ref = $settings{'Add5cell1'};
	$$add6cell1ref = $settings{'Add6cell1'};
	$$add7cell1ref = $settings{'Add7cell1'};
	$$add8cell1ref = $settings{'Add8cell1'};
	$$add9cell1ref = $settings{'Add9cell1'};
	$$addCell6ref = $settings{'AddCell6'};
	$$addCell7ref = $settings{'AddCell7'};
	$$subCell1ref = $settings{'SubCell1'};
	$$subCell2ref = $settings{'SubCell2'};
	$$subCell3ref = $settings{'SubCell3'};
	$$subCell4ref = $settings{'SubCell4'};
	$$subCell5ref = $settings{'SubCell5'};
	$$subCell6ref = $settings{'SubCell6'};
	$$subCell7ref = $settings{'SubCell7'};
	$$soundToggleref = $settings{'SoundToggle'};
	$$soundCell1ref = $settings{'SoundCell1'};
	$$soundCell2ref = $settings{'SoundCell2'};
	$$soundCell3ref = $settings{'SoundCell3'};
	$$soundCell4ref = $settings{'SoundCell4'};
	$$soundCell5ref = $settings{'SoundCell5'};
	$$soundCell6ref = $settings{'SoundCell6'};
	$$soundCell7ref = $settings{'SoundCell7'};
	$$soundAlert1ref = $settings{'SoundAlert1'};
	$$soundAlert2ref = $settings{'SoundAlert2'};
	$$soundAlert1valref = $settings{'SoundAlert1val'};
	$$soundAlert1valref = $settings{'SoundAlert2val'};
	$$equation1labelref = $settings{'Equation1label'};
	$$equation2labelref = $settings{'Equation2label'};
	$$equation1ref = $settings{'Equation1'};
	$$equation2ref = $settings{'Equation2'};
	

    }
}

sub saveSettings {
    my ($mwref,
	$cell1countref,
	$cell2Countref,
	$cell3Countref,
	$cell4Countref,
	$cell5Countref,
	$cell6countref,
	$cell7countref,
	$totalCellCountref,
	$totalCellCount_eref,
	$equation1ref,
	$equation2ref,
	$equation1labelref,
	$equation2labelref,
	$equation1val_eref,
	$equation2val_eref,
	$cell1ref,
	$cell2ref,
	$cell3ref,
	$cell4ref,
	$cell5ref,
	$cell6ref,
	$cell7ref,
	$cell1labelref,
	$cell2labelref,
	$cell3labelref,
	$cell4labelref,
	$cell5labelref,
	$cell6labelref,
	$cell7labelref,
	$cell1label_bref,
	$cell2label_bref,
	$cell3label_bref,
	$cell4label_bref,
	$cell5label_bref,
	$cell6label_bref,
	$cell7label_bref,
	$logwinref,
	$bookref,
	$keyTestWinref,
	$addCell2ref,
	$addCell3ref,
	$addCell4ref,
	$addCell5ref,
	$add1cell1ref,
	$add2cell1ref,
	$add3cell1ref,
	$add4cell1ref,
	$add5cell1ref,
	$add6cell1ref,
	$add7cell1ref,
	$add8cell1ref,
	$add9cell1ref,
	$addCell6ref,
	$addCell7ref,
	$subCell1ref,
	$subCell2ref,
	$subCell3ref,
	$subCell4ref,
	$subCell5ref,
	$subCell6ref,
	$subCell7ref,
	$soundToggleref,
	$soundCell1ref,
	$soundCell2ref,
	$soundCell3ref,
	$soundCell4ref,
	$soundCell5ref,
	$soundCell6ref,
	$soundCell7ref,
	$soundAlert1ref,
	$soundAlert2ref,
	$soundAlert1valref,
	$soundAlert2valref,
	$audioDataCell1ref,
	$audioDataCell2ref,
	$audioDataCell3ref,
	$audioDataCell4ref,
	$audioDataCell5ref,
	$audioDataCell6ref,
	$audioDataCell7ref,
	$audioDataAlert1ref,
	$audioDataAlert2ref,
	$svrref,
	) = @_;

    my $file;
    if ($^O =~ /mswin/i) {
	my $savetypes = [
			 "{All files}     *           ",
			 "{Text files}    {.txt}      ",
			 ];
	$file = $$mwref->getSaveFile(-filetypes => $savetypes,
				     -title => 'Save Settings',
				     -defaultextension => 'txt',
				     );
    }
    else {
	$file = $$mwref->getSaveFile(-title => 'Save File');
    }
    if (defined $file and $file ne '') {
	open(SAVE,">$file") or die "Couldn't save to file $file: $!\n";

	print SAVE "Cell1label\t$$cell1labelref\n";
	print SAVE "Cell2label\t$$cell2labelref\n";
	print SAVE "Cell3label\t$$cell3labelref\n";
	print SAVE "Cell4label\t$$cell4labelref\n";
	print SAVE "Cell5label\t$$cell5labelref\n";
	print SAVE "Cell6label\t$$cell6labelref\n";
	print SAVE "Cell7label\t$$cell7labelref\n";

	print SAVE "AddCell2\t$$addCell2ref\n";
	print SAVE "AddCell3\t$$addCell3ref\n";
	print SAVE "AddCell4\t$$addCell4ref\n";
	print SAVE "AddCell5\t$$addCell5ref\n";
	print SAVE "AddCell6\t$$addCell6ref\n";
	print SAVE "AddCell7\t$$addCell7ref\n";
	print SAVE "Add1cell1\t$$add1cell1ref\n";
	print SAVE "Add2cell1\t$$add2cell1ref\n";
	print SAVE "Add3cell1\t$$add3cell1ref\n";
	print SAVE "Add4cell1\t$$add4cell1ref\n";
	print SAVE "Add5cell1\t$$add5cell1ref\n";
	print SAVE "Add6cell1\t$$add6cell1ref\n";
	print SAVE "Add7cell1\t$$add7cell1ref\n";
	print SAVE "Add8cell1\t$$add8cell1ref\n";
	print SAVE "Add9cell1\t$$add9cell1ref\n";
	print SAVE "SubCell1\t$$subCell1ref\n";
	print SAVE "SubCell2\t$$subCell2ref\n";
	print SAVE "SubCell3\t$$subCell3ref\n";
	print SAVE "SubCell4\t$$subCell4ref\n";
	print SAVE "SubCell5\t$$subCell5ref\n";
	print SAVE "SubCell6\t$$subCell6ref\n";
	print SAVE "SubCell7\t$$subCell7ref\n";
	print SAVE "SoundToggle\t$$soundToggleref\n";
	print SAVE "SoundCell1\t$$soundCell1ref\n";
	print SAVE "SoundCell2\t$$soundCell2ref\n";
	print SAVE "SoundCell3\t$$soundCell3ref\n";
	print SAVE "SoundCell4\t$$soundCell4ref\n";
	print SAVE "SoundCell5\t$$soundCell5ref\n";
	print SAVE "SoundCell6\t$$soundCell6ref\n";
	print SAVE "SoundCell7\t$$soundCell7ref\n";
	print SAVE "SoundAlert1\t$$soundAlert1ref\n";
	print SAVE "SoundAlert2\t$$soundAlert2ref\n";
	print SAVE "SoundAlert1val\t$$soundAlert1valref\n";
	print SAVE "SoundAlertval\t$$soundAlert1valref\n";
	print SAVE "Equation1label\t$$equation1labelref\n";
	print SAVE "Equation2label\t$$equation2labelref\n";
	print SAVE "Equation1\t$$equation1ref\n";
	print SAVE "Equation2\t$$equation1ref\n";
	close SAVE;
    }
}

sub selectsinglefile {
    my $entryblankref = shift;
    my $entryblank = $$entryblankref;
    my $mwref = shift;
    my $mw = $$mwref;

    my $file;
    if ($^O =~ /mswin/i) {
	my $opentypes = [
			 "{All files}     *           ",
			 "{Text files}    {.txt}      ",
			 ];
	$file = $mw->getOpenFile(-filetypes=>$opentypes);
	return unless $file;
	$file =~ s/\//\\/g;
    }
    else {
	$file = $mw->getOpenFile(-title=>'Load File');
    }
    if (defined $file and $file ne '') {
	$entryblank->delete(0, 'end');
	$entryblank->insert(0, $file);
	$entryblank->xview('end');
    }
}

sub checkSettings {
    my ($mwref,
	$cell1countref,
	$cell2Countref,
	$cell3Countref,
	$cell4Countref,
	$cell5Countref,
	$cell6countref,
	$cell7countref,
	$totalCellCountref,
	$totalCellCount_eref,
	$equation1ref,
	$equation2ref,
	$equation1labelref,
	$equation2labelref,
	$equation1val_eref,
	$equation2val_eref,
	$cell1ref,
	$cell2ref,
	$cell3ref,
	$cell4ref,
	$cell5ref,
	$cell6ref,
	$cell7ref,
	$cell1labelref,
	$cell2labelref,
	$cell3labelref,
	$cell4labelref,
	$cell5labelref,
	$cell6labelref,
	$cell7labelref,
	$cell1label_bref,
	$cell2label_bref,
	$cell3label_bref,
	$cell4label_bref,
	$cell5label_bref,
	$cell6label_bref,
	$cell7label_bref,
	$logwinref,
	$bookref,
	$keyTestWinref,
	$addCell2ref,
	$addCell3ref,
	$addCell4ref,
	$addCell5ref,
	$add1cell1ref,
	$add2cell1ref,
	$add3cell1ref,
	$add4cell1ref,
	$add5cell1ref,
	$add6cell1ref,
	$add7cell1ref,
	$add8cell1ref,
	$add9cell1ref,
	$addCell6ref,
	$addCell7ref,
	$subCell1ref,
	$subCell2ref,
	$subCell3ref,
	$subCell4ref,
	$subCell5ref,
	$subCell6ref,
	$subCell7ref,
	$soundToggleref,
	$soundCell1ref,
	$soundCell2ref,
	$soundCell3ref,
	$soundCell4ref,
	$soundCell5ref,
	$soundCell6ref,
	$soundCell7ref,
	$soundAlert1ref,
	$soundAlert2ref,
	$soundAlert1valref,
	$soundAlert2valref,
	$audioDataCell1ref,
	$audioDataCell2ref,
	$audioDataCell3ref,
	$audioDataCell4ref,
	$audioDataCell5ref,
	$audioDataCell6ref,
	$audioDataCell7ref,
	$audioDataAlert1ref,
	$audioDataAlert2ref,
	$svrref,
	) = @_;

    my $currdir = getcwd;
#    print "$currdir\n";

    my @fileFlag;

    push(@fileFlag,$$soundCell1ref) unless (-e "$currdir\/$$soundCell1ref" || -e $$soundCell1ref);
    push(@fileFlag,$$soundCell2ref) unless (-e "$currdir\/$$soundCell2ref" || -e $$soundCell2ref);
    push(@fileFlag,$$soundCell3ref) unless (-e "$currdir\/$$soundCell3ref" || -e $$soundCell3ref);
    push(@fileFlag,$$soundCell4ref) unless (-e "$currdir\/$$soundCell4ref" || -e $$soundCell4ref);
    push(@fileFlag,$$soundCell5ref) unless (-e "$currdir\/$$soundCell5ref" || -e $$soundCell5ref);
    push(@fileFlag,$$soundCell6ref) unless (-e "$currdir\/$$soundCell6ref" || -e $$soundCell6ref);
    push(@fileFlag,$$soundCell7ref) unless (-e "$currdir\/$$soundCell7ref" || -e $$soundCell7ref);
    push(@fileFlag,$$soundAlert1ref) unless (-e "$currdir\/$$soundAlert1ref" || -e $$soundAlert1ref);
    push(@fileFlag,$$soundAlert2ref) unless (-e "$currdir\/$$soundAlert2ref" || -e $$soundAlert2ref);

    if (defined($fileFlag[0])) {
	my $message = "The following file\(s\) could not be loaded\:\n";
	for (0 .. $#fileFlag) {
	    $message .= "$fileFlag[$_]\n";
	}
	$$mwref->messageBox(-message => $message,
			    -type => 'OK',
			    );
    }

    if ($^O !~ /MSWin32/i) {
	# update filehandles for audio files
	open(AUDIO,$$soundCell1ref) or die "$!\n";
	my $fh = *AUDIO;
	binmode($fh);
	$$audioDataCell1ref = Audio::Data->new();
	$$audioDataCell1ref->Load($fh);
	close AUDIO;
	
	open(AUDIO,$$soundCell2ref) or die "$!\n";
	$fh = *AUDIO;
	binmode($fh);
	$$audioDataCell2ref = Audio::Data->new();
	$$audioDataCell2ref->Load($fh);
	close AUDIO;

	open(AUDIO,$$soundCell3ref) or die "$!\n";
	$fh = *AUDIO;
	binmode($fh);
	$$audioDataCell3ref = Audio::Data->new();
	$$audioDataCell3ref->Load($fh);
	close AUDIO;

	open(AUDIO,$$soundCell4ref) or die "$!\n";
	$fh = *AUDIO;
	binmode($fh);
	$$audioDataCell4ref = Audio::Data->new();
	$$audioDataCell4ref->Load($fh);
	close AUDIO;

	open(AUDIO,$$soundCell5ref) or die "$!\n";
	$fh = *AUDIO;
	binmode($fh);
	$$audioDataCell5ref = Audio::Data->new();
	$$audioDataCell5ref->Load($fh);
	close AUDIO;

	open(AUDIO,$$soundCell6ref) or die "$!\n";
	$fh = *AUDIO;
	binmode($fh);
	$$audioDataCell6ref = Audio::Data->new();
	$$audioDataCell6ref->Load($fh);
	close AUDIO;

	open(AUDIO,$$soundCell7ref) or die "$!\n";
	$fh = *AUDIO;
	binmode($fh);
	$$audioDataCell7ref = Audio::Data->new();
	$$audioDataCell7ref->Load($fh);
	close AUDIO;

	open(AUDIO,$$soundAlert1ref) or die "$!\n";
	$fh = *AUDIO;
	binmode($fh);
	$$audioDataAlert1ref = Audio::Data->new();
	$$audioDataAlert1ref->Load($fh);
	close AUDIO;

	open(AUDIO,$$soundAlert2ref) or die "$!\n";
	$fh = *AUDIO;
	binmode($fh);
	$$audioDataAlert2ref = Audio::Data->new();
	$$audioDataAlert2ref->Load($fh);
	close AUDIO;
    }
}

sub updateButtons {
    my (
	$cell1labelref,
	$cell2labelref,
	$cell3labelref,
	$cell4labelref,
	$cell5labelref,
	$cell6labelref,
	$cell7labelref,
	$cell1label_bref,
	$cell2label_bref,
	$cell3label_bref,
	$cell4label_bref,
	$cell5label_bref,
	$cell6label_bref,
	$cell7label_bref,
	$addcell1labelref,
	$addcell2labelref,
	$addcell3labelref,
	$addcell4labelref,
	$addcell5labelref,
	$addcell6labelref,
	$addcell7labelref,
	$subcell1labelref,
	$subcell2labelref,
	$subcell3labelref,
	$subcell4labelref,
	$subcell5labelref,
	$subcell6labelref,
	$subcell7labelref,
	$addCell2ref,
	$addCell3ref,
	$addCell4ref,
	$addCell5ref,
	$addCell6ref,
	$addCell7ref,
	$subCell1ref,
	$subCell2ref,
	$subCell3ref,
	$subCell4ref,
	$subCell5ref,
	$subCell6ref,
	$subCell7ref,
	$addcell2keylabelref,
	$addcell3keylabelref,
	$addcell4keylabelref,
	$addcell5keylabelref,
	$addcell6keylabelref,
	$addcell7keylabelref,
	$subcell1keylabelref,
	$subcell2keylabelref,
	$subcell3keylabelref,
	$subcell4keylabelref,
	$subcell5keylabelref,
	$subcell6keylabelref,
	$subcell7keylabelref,
	$equation1label_eref,
	$equation2label_eref,
	$equation1labelref,
	$equation2labelref
	) = @_;

    my $maxWidth = 1;
    $maxWidth = length($$cell1labelref) if length($$cell1labelref) > $maxWidth;
    $maxWidth = length($$cell2labelref) if length($$cell2labelref) > $maxWidth;
    $maxWidth = length($$cell3labelref) if length($$cell3labelref) > $maxWidth;
    $maxWidth = length($$cell4labelref) if length($$cell4labelref) > $maxWidth;
    $maxWidth = length($$cell5labelref) if length($$cell5labelref) > $maxWidth;
    $maxWidth = length($$cell6labelref) if length($$cell6labelref) > $maxWidth;
    $maxWidth = length($$cell7labelref) if length($$cell7labelref) > $maxWidth;
#    print "$maxWidth\n";

    $$cell1label_bref->configure(-text => $$cell1labelref,
				 -width => $maxWidth);
    $$cell1label_bref->update;
    $$cell2label_bref->configure(-text => $$cell2labelref,
				 -width => $maxWidth);
    $$cell2label_bref->update;
    $$cell3label_bref->configure(-text => $$cell3labelref,
				 -width => $maxWidth);
    $$cell3label_bref->update;
    $$cell4label_bref->configure(-text => $$cell4labelref,
				 -width => $maxWidth);
    $$cell4label_bref->update;
    $$cell5label_bref->configure(-text => $$cell5labelref,
				 -width => $maxWidth);
    $$cell5label_bref->update;
    $$cell6label_bref->configure(-text => $$cell6labelref,
				 -width => $maxWidth);
    $$cell6label_bref->update;
    $$cell7label_bref->configure(-text => $$cell7labelref,
				 -width => $maxWidth);
    $$cell7label_bref->update;

    $$addcell1labelref->configure(-text => "Add 1-9 $$cell1labelref");
    $$addcell1labelref->update;
    $$addcell2labelref->configure(-text => "Add 1 $$cell2labelref");
    $$addcell2labelref->update;
    $$addcell3labelref->configure(-text => "Add 1 $$cell3labelref");
    $$addcell3labelref->update;
    $$addcell4labelref->configure(-text => "Add 1 $$cell4labelref");
    $$addcell4labelref->update;
    $$addcell5labelref->configure(-text => "Add 1 $$cell5labelref");
    $$addcell5labelref->update;
    $$addcell6labelref->configure(-text => "Add 1 $$cell6labelref");
    $$addcell6labelref->update;
    $$addcell7labelref->configure(-text => "Add 1 $$cell7labelref");
    $$addcell7labelref->update;

    $$subcell1labelref->configure(-text => "Sub 1 $$cell1labelref");
    $$subcell1labelref->update;
    $$subcell2labelref->configure(-text => "Sub 1 $$cell2labelref");
    $$subcell2labelref->update;
    $$subcell3labelref->configure(-text => "Sub 1 $$cell3labelref");
    $$subcell3labelref->update;
    $$subcell4labelref->configure(-text => "Sub 1 $$cell4labelref");
    $$subcell4labelref->update;
    $$subcell5labelref->configure(-text => "Sub 1 $$cell5labelref");
    $$subcell5labelref->update;
    $$subcell6labelref->configure(-text => "Sub 1 $$cell6labelref");
    $$subcell6labelref->update;
    $$subcell7labelref->configure(-text => "Sub 1 $$cell7labelref");
    $$subcell7labelref->update;

    $$addcell2keylabelref->configure(-text => $$addCell2ref);
    $$addcell2keylabelref->update;
    $$addcell3keylabelref->configure(-text => $$addCell3ref);
    $$addcell3keylabelref->update;
    $$addcell4keylabelref->configure(-text => $$addCell4ref);
    $$addcell4keylabelref->update;
    $$addcell5keylabelref->configure(-text => $$addCell5ref);
    $$addcell5keylabelref->update;
    $$addcell6keylabelref->configure(-text => $$addCell6ref);
    $$addcell6keylabelref->update;
    $$addcell7keylabelref->configure(-text => $$addCell7ref);
    $$addcell7keylabelref->update;

    $$subcell1keylabelref->configure(-text => $$subCell1ref);
    $$subcell1keylabelref->update;
    $$subcell2keylabelref->configure(-text => $$subCell2ref);
    $$subcell2keylabelref->update;
    $$subcell3keylabelref->configure(-text => $$subCell3ref);
    $$subcell3keylabelref->update;
    $$subcell4keylabelref->configure(-text => $$subCell4ref);
    $$subcell4keylabelref->update;
    $$subcell5keylabelref->configure(-text => $$subCell5ref);
    $$subcell5keylabelref->update;
    $$subcell6keylabelref->configure(-text => $$subCell6ref);
    $$subcell6keylabelref->update;
    $$subcell7keylabelref->configure(-text => $$subCell7ref);
    $$subcell7keylabelref->update;

    if ($$equation1label_eref) {
	$$equation1label_eref->configure(-text => $$equation1labelref);
	$$equation1label_eref->update;
    }
    if ($$equation2label_eref) {
	$$equation2label_eref->configure(-text => $$equation2labelref);
	$$equation2label_eref->update;
    }

}


sub autoSaveOnExit {
    my ($mwref,
	$cell1countref,
	$cell2Countref,
	$cell3Countref,
	$cell4Countref,
	$cell5Countref,
	$cell6countref,
	$cell7countref,
	$totalCellCountref,
	$totalCellCount_eref,
	$equation1ref,
	$equation2ref,
	$equation1labelref,
	$equation2labelref,
	$equation1val_eref,
	$equation2val_eref,
	$cell1ref,
	$cell2ref,
	$cell3ref,
	$cell4ref,
	$cell5ref,
	$cell6ref,
	$cell7ref,
	$cell1labelref,
	$cell2labelref,
	$cell3labelref,
	$cell4labelref,
	$cell5labelref,
	$cell6labelref,
	$cell7labelref,
	$cell1label_bref,
	$cell2label_bref,
	$cell3label_bref,
	$cell4label_bref,
	$cell5label_bref,
	$cell6label_bref,
	$cell7label_bref,
	$logwinref,
	$bookref,
	$keyTestWinref,
	$addCell2ref,
	$addCell3ref,
	$addCell4ref,
	$addCell5ref,
	$add1cell1ref,
	$add2cell1ref,
	$add3cell1ref,
	$add4cell1ref,
	$add5cell1ref,
	$add6cell1ref,
	$add7cell1ref,
	$add8cell1ref,
	$add9cell1ref,
	$addCell6ref,
	$addCell7ref,
	$subCell1ref,
	$subCell2ref,
	$subCell3ref,
	$subCell4ref,
	$subCell5ref,
	$subCell6ref,
	$subCell7ref,
	$soundToggleref,
	$soundCell1ref,
	$soundCell2ref,
	$soundCell3ref,
	$soundCell4ref,
	$soundCell5ref,
	$soundCell6ref,
	$soundCell7ref,
	$soundAlert1ref,
	$soundAlert2ref,
	$soundAlert1valref,
	$soundAlert2valref,
	$audioDataCell1ref,
	$audioDataCell2ref,
	$audioDataCell3ref,
	$audioDataCell4ref,
	$audioDataCell5ref,
	$audioDataCell6ref,
	$audioDataCell7ref,
	$audioDataAlert1ref,
	$audioDataAlert2ref,
	$svrref,
	) = @_;

    unless (-e 'VersaCountSettings') {
	mkdir('VersaCountSettings');
    }

    open(SAVE,">VersaCountSettings\/autoSaveSettings.txt") or die "Couldn't auto-save settings: $!\n";

    print SAVE "Cell1label\t$$cell1labelref\n";
    print SAVE "Cell2label\t$$cell2labelref\n";
    print SAVE "Cell3label\t$$cell3labelref\n";
    print SAVE "Cell4label\t$$cell4labelref\n";
    print SAVE "Cell5label\t$$cell5labelref\n";
    print SAVE "Cell6label\t$$cell6labelref\n";
    print SAVE "Cell7label\t$$cell7labelref\n";

    print SAVE "AddCell2\t$$addCell2ref\n";
    print SAVE "AddCell3\t$$addCell3ref\n";
    print SAVE "AddCell4\t$$addCell4ref\n";
    print SAVE "AddCell5\t$$addCell5ref\n";
    print SAVE "AddCell6\t$$addCell6ref\n";
    print SAVE "AddCell7\t$$addCell7ref\n";
    print SAVE "Add1cell1\t$$add1cell1ref\n";
    print SAVE "Add2cell1\t$$add2cell1ref\n";
    print SAVE "Add3cell1\t$$add3cell1ref\n";
    print SAVE "Add4cell1\t$$add4cell1ref\n";
    print SAVE "Add5cell1\t$$add5cell1ref\n";
    print SAVE "Add6cell1\t$$add6cell1ref\n";
    print SAVE "Add7cell1\t$$add7cell1ref\n";
    print SAVE "Add8cell1\t$$add8cell1ref\n";
    print SAVE "Add9cell1\t$$add9cell1ref\n";
    print SAVE "SubCell1\t$$subCell1ref\n";
    print SAVE "SubCell2\t$$subCell2ref\n";
    print SAVE "SubCell3\t$$subCell3ref\n";
    print SAVE "SubCell4\t$$subCell4ref\n";
    print SAVE "SubCell5\t$$subCell5ref\n";
    print SAVE "SubCell6\t$$subCell6ref\n";
    print SAVE "SubCell7\t$$subCell7ref\n";
    print SAVE "SoundToggle\t$$soundToggleref\n";
    print SAVE "SoundCell1\t$$soundCell1ref\n";
    print SAVE "SoundCell2\t$$soundCell2ref\n";
    print SAVE "SoundCell3\t$$soundCell3ref\n";
    print SAVE "SoundCell4\t$$soundCell4ref\n";
    print SAVE "SoundCell5\t$$soundCell5ref\n";
    print SAVE "SoundCell6\t$$soundCell6ref\n";
    print SAVE "SoundCell7\t$$soundCell7ref\n";
    print SAVE "SoundAlert1\t$$soundAlert1ref\n";
    print SAVE "SoundAlert2\t$$soundAlert2ref\n";
    print SAVE "SoundAlert1val\t$$soundAlert1valref\n";
    print SAVE "SoundAlert2val\t$$soundAlert2valref\n";
    print SAVE "Equation1label\t$$equation1labelref\n";
    print SAVE "Equation2label\t$$equation2labelref\n";
    print SAVE "Equation1\t$$equation1ref\n";
    print SAVE "Equation2\t$$equation2ref\n";
    close SAVE;
}

sub autoLoadSettings {
    my ($mwref,
	$cell1countref,
	$cell2Countref,
	$cell3Countref,
	$cell4Countref,
	$cell5Countref,
	$cell6countref,
	$cell7countref,
	$totalCellCountref,
	$totalCellCount_eref,
	$equation1ref,
	$equation2ref,
	$equation1labelref,
	$equation2labelref,
	$equation1val_eref,
	$equation2val_eref,
	$cell1ref,
	$cell2ref,
	$cell3ref,
	$cell4ref,
	$cell5ref,
	$cell6ref,
	$cell7ref,
	$cell1labelref,
	$cell2labelref,
	$cell3labelref,
	$cell4labelref,
	$cell5labelref,
	$cell6labelref,
	$cell7labelref,
	$cell1label_bref,
	$cell2label_bref,
	$cell3label_bref,
	$cell4label_bref,
	$cell5label_bref,
	$cell6label_bref,
	$cell7label_bref,
	$logwinref,
	$bookref,
	$keyTestWinref,
	$addCell2ref,
	$addCell3ref,
	$addCell4ref,
	$addCell5ref,
	$add1cell1ref,
	$add2cell1ref,
	$add3cell1ref,
	$add4cell1ref,
	$add5cell1ref,
	$add6cell1ref,
	$add7cell1ref,
	$add8cell1ref,
	$add9cell1ref,
	$addCell6ref,
	$addCell7ref,
	$subCell1ref,
	$subCell2ref,
	$subCell3ref,
	$subCell4ref,
	$subCell5ref,
	$subCell6ref,
	$subCell7ref,
	$soundToggleref,
	$soundCell1ref,
	$soundCell2ref,
	$soundCell3ref,
	$soundCell4ref,
	$soundCell5ref,
	$soundCell6ref,
	$soundCell7ref,
	$soundAlert1ref,
	$soundAlert2ref,
	$soundAlert1valref,
	$soundAlert2valref,
	$audioDataCell1ref,
	$audioDataCell2ref,
	$audioDataCell3ref,
	$audioDataCell4ref,
	$audioDataCell5ref,
	$audioDataCell6ref,
	$audioDataCell75ref,
	$audioDataAlert1ref,
	$audioDataAlert2ref,
	$svrref,
	) = @_;

    my $file = "VersaCountSettings\/autoSaveSettings.txt";

    unless (-e "VersaCountSettings") {
	mkdir("VersaCountSettings");
    }
    unless (-e $file) {
	return;
    }

    # read settings from file
    open(LOAD,$file) or die "Couldn't open file $file: $!\n";
    my %settings;
    while(<LOAD>) {
	chomp;
	$_ =~ s/\r//g;
	my @line = split(/\t/);
	$settings{$line[0]} = $line[1];
    }
    close LOAD;
    
    # assign settings to variables

    $$cell1labelref = $settings{'Cell1label'};
    $$cell2labelref = $settings{'Cell2label'};
    $$cell3labelref = $settings{'Cell3label'};
    $$cell4labelref = $settings{'Cell4label'};
    $$cell5labelref = $settings{'Cell5label'};
    $$cell6labelref = $settings{'Cell6label'};
    $$cell7labelref = $settings{'Cell7label'};

    $$addCell2ref = $settings{'AddCell2'};
    $$addCell3ref = $settings{'AddCell3'};
    $$addCell4ref = $settings{'AddCell4'};
    $$addCell5ref = $settings{'AddCell5'};
    $$add1cell1ref = $settings{'Add1cell1'};
    $$add2cell1ref = $settings{'Add2cell1'};
    $$add3cell1ref = $settings{'Add3cell1'};
    $$add4cell1ref = $settings{'Add4cell1'};
    $$add5cell1ref = $settings{'Add5cell1'};
    $$add6cell1ref = $settings{'Add6cell1'};
    $$add7cell1ref = $settings{'Add7cell1'};
    $$add8cell1ref = $settings{'Add8cell1'};
    $$add9cell1ref = $settings{'Add9cell1'};
    $$addCell6ref = $settings{'AddCell6'};
    $$addCell7ref = $settings{'AddCell7'};
    $$subCell1ref = $settings{'SubCell1'};
    $$subCell2ref = $settings{'SubCell2'};
    $$subCell3ref = $settings{'SubCell3'};
    $$subCell4ref = $settings{'SubCell4'};
    $$subCell5ref = $settings{'SubCell5'};
    $$subCell6ref = $settings{'SubCell6'};
    $$subCell7ref = $settings{'SubCell7'};
    $$soundToggleref = $settings{'SoundToggle'};
    $$soundCell1ref = $settings{'SoundCell1'};
    $$soundCell2ref = $settings{'SoundCell2'};
    $$soundCell3ref = $settings{'SoundCell3'};
    $$soundCell4ref = $settings{'SoundCell4'};
    $$soundCell5ref = $settings{'SoundCell5'};
    $$soundCell6ref = $settings{'SoundCell6'};
    $$soundCell7ref = $settings{'SoundCell7'};
    $$soundAlert1ref = $settings{'SoundAlert1'};
    $$soundAlert2ref = $settings{'SoundAlert2'};
    $$soundAlert1valref = $settings{'SoundAlert1val'};
    $$soundAlert2valref = $settings{'SoundAlert2val'};
    $$equation1labelref = $settings{'Equation1label'};
    $$equation2labelref = $settings{'Equation2label'};
    $$equation1ref = $settings{'Equation1'};
    $$equation2ref = $settings{'Equation2'};
}
